@@ -50,7 +50,7 @@
   #+clisp `(let ((custom:*suppress-check-redefinition* t)) ,form)
   #-(or sbcl clisp) `(progn ,form))
 
-($put '$wxmaxima `((mlist simp) 15 04 0) '$version)
+($put '$wxmaxima `((mlist simp) 15 8 2) '$version)
 
 (defun $wxbuild_info ()
   (let ((wxmaxima-version (cdr ($get '$wxmaxima '$version)))
@@ -271,7 +271,7 @@
 
 (defun wxxml-nary (x l r)
   (let* ((op (caar x))
-	 (sym (cond ((eq op 'mtimes)
+	 (sym (cond ((member op '(mtimes wxtimes) :test #'eq)
 		     (if $stardisp
 			 "<t>*</t>"
 			 "<h>*</h>"))
@@ -606,6 +606,12 @@
 (defprop mtimes 120. wxxml-lbp)
 (defprop mtimes 120. wxxml-rbp)
 
+(defprop wxtimes wxxml-nary wxxml)
+(defprop wxtimes "<h>*</h>" wxxmlsym)
+(defprop wxtimes "<t>*</t>" wxxmlword)
+(defprop wxtimes 120. wxxml-lbp)
+(defprop wxtimes 120. wxxml-rbp)
+
 (defprop %sqrt wxxml-sqrt wxxml)
 
 (defun wxxml-sqrt (x l r)
@@ -937,7 +943,7 @@
                     (mapcan #'(lambda(b e)
                                 `(,dsym ,(simplifya `((mexpt) ,b ,e) nil)))
                             vars ords))))
-    `((mtimes)
+    `((wxtimes)
       ((mquotient) ,(simplifya numer nil) ,denom)
       ,arg)))
 
@@ -1461,8 +1467,9 @@
 	 res)
     (setq res ($apply '$draw
                       (append
-                       `((mlist simp)
-			 ((mequal simp) $gnuplot_file_name ,$gnuplot_file_name)
+                       '((mlist simp))
+                       args
+                       `(((mequal simp) $gnuplot_file_name ,$gnuplot_file_name)
 			 ((mequal simp) $data_file_name ,$data_file_name)
                          ((mequal simp) $terminal ,(if $wxplot_pngcairo '$pngcairo '$png))
                          ((mequal simp) $file_name ,filename))
@@ -1471,8 +1478,7 @@
                           `(((mequal simp) $pic_width ,($first $wxplot_size))
                             ((mequal simp) $pic_height ,($second $wxplot_size))))
                          (t
-                          `(((mequal simp) $dimensions ,$wxplot_size))))
-                       args)))
+                          `(((mequal simp) $dimensions ,$wxplot_size)))))))
     (if $display_graphics
 	(progn
           ($ldisp `((wxxmltag simp) ,(format nil "~a.png" filename) "img"))
