@@ -85,7 +85,7 @@ CellParser::~CellParser()
 
 wxString CellParser::GetFontName(int type)
 {
-  if (type == TS_TITLE || type == TS_SUBSECTION || type == TS_SECTION || type == TS_TEXT)
+  if (type == TS_TITLE || type == TS_SUBSECTION || type == TS_SUBSUBSECTION || type == TS_SECTION || type == TS_TEXT)
     return m_styles[type].font;
   else if (type == TS_NUMBER || type == TS_VARIABLE || type == TS_FUNCTION ||
       type == TS_SPECIAL_CONSTANT || type == TS_STRING)
@@ -126,7 +126,7 @@ void CellParser::ReadStyle()
 
   // Normal text
   m_styles[TS_DEFAULT].color = wxT("black");
-  m_styles[TS_DEFAULT].bold = false;
+  m_styles[TS_DEFAULT].bold = true;
   m_styles[TS_DEFAULT].italic = true;
   m_styles[TS_DEFAULT].underlined = false;
   READ_STYLES(TS_DEFAULT, "Style/NormalText/")
@@ -143,6 +143,67 @@ void CellParser::ReadStyle()
                &m_styles[TS_TEXT].font);
   READ_STYLES(TS_TEXT, "Style/Text/")
 
+  // Variables in highlighted code
+  m_styles[TS_CODE_VARIABLE].color = wxT("rgb(0,128,0)");
+  m_styles[TS_CODE_VARIABLE].bold = false;
+  m_styles[TS_CODE_VARIABLE].italic = true;
+  m_styles[TS_CODE_VARIABLE].underlined = false;
+  READ_STYLES(TS_CODE_VARIABLE, "Style/CodeHighlighting/Variable/")
+
+  // Keywords in highlighted code
+  m_styles[TS_CODE_FUNCTION].color = wxT("rgb(128,0,0)");
+  m_styles[TS_CODE_FUNCTION].bold = false;
+  m_styles[TS_CODE_FUNCTION].italic = true;
+  m_styles[TS_CODE_FUNCTION].underlined = false;
+  READ_STYLES(TS_CODE_FUNCTION, "Style/CodeHighlighting/Function/")
+
+  // Comments in highlighted code
+  m_styles[TS_CODE_COMMENT].color = wxT("rgb(64,64,64)");
+  m_styles[TS_CODE_COMMENT].bold = false;
+  m_styles[TS_CODE_COMMENT].italic = true;
+  m_styles[TS_CODE_COMMENT].underlined = false;
+  READ_STYLES(TS_CODE_COMMENT, "Style/CodeHighlighting/Comment/")
+
+  // Numbers in highlighted code
+  m_styles[TS_CODE_NUMBER].color = wxT("rgb(128,64,0)");
+  m_styles[TS_CODE_NUMBER].bold = false;
+  m_styles[TS_CODE_NUMBER].italic = true;
+  m_styles[TS_CODE_NUMBER].underlined = false;
+  READ_STYLES(TS_CODE_NUMBER, "Style/CodeHighlighting/Number/")
+
+  // Strings in highlighted code
+  m_styles[TS_CODE_STRING].color = wxT("rgb(0,0,128)");
+  m_styles[TS_CODE_STRING].bold = false;
+  m_styles[TS_CODE_STRING].italic = true;
+  m_styles[TS_CODE_STRING].underlined = false;
+  READ_STYLES(TS_CODE_STRING, "Style/CodeHighlighting/String/")
+
+  // Operators in highlighted code
+  m_styles[TS_CODE_OPERATOR].color = wxT("rgb(0,0,0)");
+  m_styles[TS_CODE_OPERATOR].bold = false;
+  m_styles[TS_CODE_OPERATOR].italic = true;
+  m_styles[TS_CODE_OPERATOR].underlined = false;
+  READ_STYLES(TS_CODE_OPERATOR, "Style/CodeHighlighting/Operator/")
+    
+  // Line endings in highlighted code
+  m_styles[TS_CODE_ENDOFLINE].color = wxT("rgb(192,192,192)");
+  m_styles[TS_CODE_ENDOFLINE].bold = false;
+  m_styles[TS_CODE_ENDOFLINE].italic = true;
+  m_styles[TS_CODE_ENDOFLINE].underlined = false;
+  READ_STYLES(TS_CODE_ENDOFLINE, "Style/CodeHighlighting/EndOfLine/")
+    
+  // Subsubsection
+  m_styles[TS_SUBSUBSECTION].color = wxT("black");
+  m_styles[TS_SUBSUBSECTION].bold = true;
+  m_styles[TS_SUBSUBSECTION].italic = false;
+  m_styles[TS_SUBSUBSECTION].underlined = false;
+  m_styles[TS_SUBSUBSECTION].fontSize = 14;
+  config->Read(wxT("Style/Subsubsection/fontsize"),
+               &m_styles[TS_SUBSUBSECTION].fontSize);
+  config->Read(wxT("Style/Subsubsection/fontname"),
+               &m_styles[TS_SUBSUBSECTION].font);
+  READ_STYLES(TS_SUBSUBSECTION, "Style/Subsubsection/")
+
   // Subsection
   m_styles[TS_SUBSECTION].color = wxT("black");
   m_styles[TS_SUBSECTION].bold = true;
@@ -255,7 +316,7 @@ void CellParser::ReadStyle()
                    &tmp)) m_styles[TS_HIGHLIGHT].color.Set(tmp);
 
   // Text background
-  m_styles[TS_TEXT_BACKGROUND].color = wxColour(wxT("light blue"));
+  m_styles[TS_TEXT_BACKGROUND].color = wxColour(wxT("white"));
   if (config->Read(wxT("Style/TextBackground/color"),
                    &tmp)) m_styles[TS_TEXT_BACKGROUND].color.Set(tmp);
 
@@ -266,7 +327,7 @@ void CellParser::ReadStyle()
 
   m_styles[TS_ACTIVE_CELL_BRACKET].color = wxT("rgb(255,0,0)");
   if (config->Read(wxT("Style/ActiveCellBracket/color"),
-                   &tmp)) m_styles[TS_ACTIVE_CELL_BRACKET].color.Set(tmp);
+                  &tmp)) m_styles[TS_ACTIVE_CELL_BRACKET].color.Set(tmp);
 
   // Cursor (hcaret in MathCtrl and caret in EditorCell)
   m_styles[TS_CURSOR].color = wxT("rgb(0,0,0)");
@@ -281,6 +342,9 @@ void CellParser::ReadStyle()
 #endif
   if (config->Read(wxT("Style/Selection/color"),
                    &tmp)) m_styles[TS_SELECTION].color.Set(tmp);
+  m_styles[TS_EQUALSSELECTION].color = wxT("rgb(192,192,255)");
+  if (config->Read(wxT("Style/EqualsSelection/color"),
+                   &tmp)) m_styles[TS_EQUALSSELECTION].color.Set(tmp);
 
   // Outdated cells
   m_styles[TS_OUTDATED].color = wxT("rgb(153,153,153)");
