@@ -92,6 +92,7 @@ Config::Config(wxWindow* parent)
   m_imageList->Add(IMAGE("editing.png"));
   m_imageList->Add(IMAGE("maxima.png"));
   m_imageList->Add(IMAGE("styles.png"));
+  m_imageList->Add(IMAGE("Document-export.png"));
   m_imageList->Add(IMAGE("options.png"));
 
   Create(parent, wxID_ANY, _("wxMaxima configuration"),
@@ -104,7 +105,8 @@ Config::Config(wxWindow* parent)
   m_notebook->AddPage(CreateWorksheetPanel(), _("Worksheet"), true, 0);
   m_notebook->AddPage(CreateMaximaPanel(), _("Maxima"), false, 1);
   m_notebook->AddPage(CreateStylePanel(), _("Style"), false, 2);
-  m_notebook->AddPage(CreateOptionsPanel(), _("Options"), false, 3);
+  m_notebook->AddPage(CreateExportPanel(), _("Export"), false, 3);
+  m_notebook->AddPage(CreateOptionsPanel(), _("Options"), false, 4);
 
 #ifndef __WXMAC__
   CreateButtons(wxOK | wxCANCEL);
@@ -126,6 +128,8 @@ void Config::SetProperties()
 {
   SetTitle(_("wxMaxima configuration"));
 
+  m_abortOnError->SetToolTip(_("If multiple cells are evaluated in one go: Abort evaluation if wxMaxima detects that maxima has encountered any error."));
+  m_pollStdOut->SetToolTip(_("Once the local network link between maxima and wxMaxima has been established maxima has no reason to send any messages using the system's stdout stream so all this stream transport should be a greeting message; The lisp running maxima will send eventual error messages using the system's stderr stream instead. If this box is checked we will nonetheless watch maxima's stdout stream for messages."));
   m_maximaProgram->SetToolTip(_("Enter the path to the Maxima executable."));
   m_additionalParameters->SetToolTip(_("Additional parameters for Maxima"
                                        " (e.g. -l clisp)."));
@@ -140,11 +144,16 @@ void Config::SetProperties()
   m_AnimateLaTeX->SetToolTip(_("Some PDF viewers are able to display moving images and wxMaxima is able to output them. If this option is selected additional LaTeX packages might be needed in order to compile the output, though."));
   m_TeXExponentsAfterSubscript->SetToolTip(_("In the LaTeX output: Put exponents after an eventual subscript instead of above it. Might increase readability for some fonts and short subscripts."));
   m_flowedTextRequested->SetToolTip(_("While text cells in LaTeX are broken into lines by TeX the text displayed on the screen is broken into lines manually. This option, if set tells that lines in HTML output will be broken where they are broken in the worksheet. If this option isn't set manual linebreaks can still be introduced by introducing an empty line."));
+  m_bitmapScale->SetToolTip(_("Normally html expects images to be rather low-res but space saving. These images tend to look rather blurry when viewed on modern screens. Therefore this setting was introduces that selects the factor by which the HTML export increases the resolution in respect to the default value."));
+  m_exportInput->SetToolTip(_("Normally we export the whole worksheet to TeX or HTML. But sometimes the maxima input does scare the user. This option turns off exporting of maxima's input."));
+  m_exportContainsWXMX->SetToolTip(_("If this option is set the .wxmx source of the current file is copied to a place a link to is put into the result of an export."));
+  m_exportWithMathJAX->SetToolTip(_("Use MathJAX instead of images in HTML exports to display maxima output. The advantage of MathJAX is that it allows to copy the displayed equations as if they were text, to choose if they should be copied as TeX or MathML instead and displays them in a scaleable format that is really nice to look at. The disadvantage of MathJAX is that it will need JavaScript and a little bit of time in order to typeset an equation."));
   m_savePanes->SetToolTip(_("Save panes layout between sessions."));
   m_usepngCairo->SetToolTip(_("The pngCairo terminal offers much better graphics quality (antialiassing and additional line styles). But it will only produce plots if the gnuplot installed on the current system actually supports it."));
   m_matchParens->SetToolTip(_("Write matching parenthesis in text controls."));
-  m_showLong->SetToolTip(_("Show long expressions in wxMaxima document."));
+  m_showLength->SetToolTip(_("Show long expressions in wxMaxima document."));
   m_language->SetToolTip(_("Language used for wxMaxima GUI."));
+  m_documentclass->SetToolTip(_("The document class LaTeX is instructed to use for our documents."));
   m_fixedFontInTC->SetToolTip(_("Set fixed font in text controls."));
   m_getFont->SetToolTip(_("Font used for display in document."));
   m_getMathFont->SetToolTip(_("Font used for displaying math characters in document."));
@@ -163,21 +172,27 @@ void Config::SetProperties()
 
   // The default values for all config items that will be used if there is no saved
   // configuration data for this item.
-  bool match = true, showLongExpr = false, savePanes = false, UncompressedWXMX=true;
-  bool fixedFontTC = true, changeAsterisk = false, usejsmath = true, keepPercent = true;
-  bool enterEvaluates = false, saveUntitled = true, openHCaret = false, AnimateLaTeX = true, TeXExponentsAfterSubscript=false, flowedTextRequested = true;
+  bool match = true, savePanes = false, UncompressedWXMX=true;
+  bool fixedFontTC = true, changeAsterisk = false, usejsmath = true, keepPercent = true, abortOnError = true, pollStdOut = false;
+  bool enterEvaluates = false, saveUntitled = true,
+    openHCaret = false, AnimateLaTeX = true, TeXExponentsAfterSubscript=false,
+    flowedTextRequested = true, exportInput = true, exportContainsWXMX = false,
+    exportWithMathJAX = true;
   bool insertAns = true;
   int  undoLimit = 0;
+  int showLength = 0;
+  int  bitmapScale = 3;
   bool fixReorderedIndices = false;
   int defaultFramerate = 2;
   int displayedDigits = 100;
   wxString texPreamble=wxEmptyString;
+  wxString documentclass=wxT("article");
   int autoSaveInterval = 0;
   
 #if defined (__WXMAC__)
-  bool usepngCairo=true;
-#else
   bool usepngCairo=false;
+#else
+  bool usepngCairo=true;
 #endif
 
   
@@ -199,10 +214,14 @@ void Config::SetProperties()
   config->Read(wxT("AnimateLaTeX"), &AnimateLaTeX);
   config->Read(wxT("TeXExponentsAfterSubscript"), &TeXExponentsAfterSubscript);
   config->Read(wxT("flowedTextRequested"), &flowedTextRequested);
+  config->Read(wxT("exportInput"), &exportInput);
+  config->Read(wxT("exportContainsWXMX"), &exportContainsWXMX);
+  config->Read(wxT("exportWithMathJAX"), &exportWithMathJAX);
   config->Read(wxT("pos-restore"), &rs);
   config->Read(wxT("matchParens"), &match);
-  config->Read(wxT("showLong"), &showLongExpr);
+  config->Read(wxT("showLength"), &showLength);
   config->Read(wxT("language"), &lang);
+  config->Read(wxT("documentclass"), &documentclass);
   config->Read(wxT("texPreamble"), &texPreamble);
   config->Read(wxT("autoSaveInterval"), &autoSaveInterval);
   config->Read(wxT("changeAsterisk"), &changeAsterisk);
@@ -213,10 +232,12 @@ void Config::SetProperties()
   config->Read(wxT("openHCaret"), &openHCaret);
   config->Read(wxT("insertAns"), &insertAns);
   config->Read(wxT("undoLimit"), &undoLimit);
+  config->Read(wxT("bitmapScale"), &bitmapScale);
   config->Read(wxT("fixReorderedIndices"), &fixReorderedIndices);
   config->Read(wxT("usejsmath"), &usejsmath);
   config->Read(wxT("keepPercent"), &keepPercent);
-
+  config->Read(wxT("abortOnError"), &abortOnError);
+  config->Read(wxT("pollStdOut"), &pollStdOut);
   unsigned int i = 0;
   for (i = 0; i < LANGUAGE_NUMBER; i++)
     if (langs[i] == lang)
@@ -226,6 +247,7 @@ void Config::SetProperties()
   else
     m_language->SetSelection(0);
 
+  m_documentclass->SetValue(documentclass);
   m_texPreamble->SetValue(texPreamble);
   m_autoSaveInterval->SetValue(autoSaveInterval);
 
@@ -257,18 +279,24 @@ void Config::SetProperties()
   m_AnimateLaTeX->SetValue(AnimateLaTeX);
   m_TeXExponentsAfterSubscript->SetValue(TeXExponentsAfterSubscript);
   m_flowedTextRequested->SetValue(flowedTextRequested);
+  m_exportInput->SetValue(exportInput);
+  m_exportContainsWXMX->SetValue(exportContainsWXMX);
+  m_exportWithMathJAX->SetValue(exportWithMathJAX);
   m_matchParens->SetValue(match);
-  m_showLong->SetValue(showLongExpr);
+  m_showLength->SetSelection(showLength);
   m_changeAsterisk->SetValue(changeAsterisk);
   m_enterEvaluates->SetValue(enterEvaluates);
   m_saveUntitled->SetValue(saveUntitled);
   m_openHCaret->SetValue(openHCaret);
   m_insertAns->SetValue(insertAns);
   m_undoLimit->SetValue(undoLimit);
+  m_bitmapScale->SetValue(bitmapScale);
   m_fixReorderedIndices->SetValue(fixReorderedIndices);
   m_fixedFontInTC->SetValue(fixedFontTC);
   m_useJSMath->SetValue(usejsmath);
   m_keepPercentWithSpecials->SetValue(keepPercent);
+  m_abortOnError->SetValue(abortOnError);
+  m_pollStdOut->SetValue(pollStdOut);
   m_defaultFramerate->SetValue(defaultFramerate);
   m_defaultPlotWidth->SetValue(defaultPlotWidth);
   m_defaultPlotHeight->SetValue(defaultPlotHeight);
@@ -290,7 +318,9 @@ wxPanel* Config::CreateWorksheetPanel()
 {
   wxPanel *panel = new wxPanel(m_notebook, -1);
 
-  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(4, 2, 5, 5);
+  wxArrayString showLengths;
+
+  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(6, 2, 5, 5);
   wxFlexGridSizer* vsizer = new wxFlexGridSizer(16,1,5,5);
   
   wxStaticText* df = new wxStaticText(panel, -1, _("Default animation framerate:"));
@@ -321,15 +351,26 @@ wxPanel* Config::CreateWorksheetPanel()
   grid_sizer->Add(ul, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer->Add(m_undoLimit, 0, wxALL, 5);
 
+  wxStaticText* bs = new wxStaticText(panel, -1, _("Bitmap scale for export"));
+  m_bitmapScale = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(100, -1), wxSP_ARROW_KEYS, 1, 3);
+  grid_sizer->Add(bs, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  grid_sizer->Add(m_bitmapScale, 0, wxALL, 5);
+
+  wxStaticText* sl = new wxStaticText(panel, -1, _("Show long expressions"));
+  grid_sizer->Add(sl, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  showLengths.Add(_("No"));
+  showLengths.Add(_("If not very long"));
+  showLengths.Add(_("If not extremely long"));
+  showLengths.Add(_("Yes"));
+  m_showLength = new wxChoice(panel,-1,wxDefaultPosition,wxDefaultSize,showLengths);
+  grid_sizer->Add(m_showLength, 0, wxALL, 5);
+
   m_matchParens = new wxCheckBox(panel, -1, _("Match parenthesis in text controls"));
   vsizer->Add(m_matchParens, 0, wxALL, 5);
 
   m_fixedFontInTC = new wxCheckBox(panel, -1, _("Fixed font in text controls"));
   vsizer->Add(m_fixedFontInTC, 0, wxALL, 5);
 
-  m_showLong = new wxCheckBox(panel, -1, _("Show long expressions"));
-  vsizer->Add(m_showLong, 0, wxALL, 5);
-
   m_changeAsterisk = new wxCheckBox(panel, -1, _("Use centered dot character for multiplication"));
   vsizer->Add(m_changeAsterisk, 0, wxALL, 5);
 
@@ -352,11 +393,55 @@ wxPanel* Config::CreateWorksheetPanel()
   return panel;
 }
   
+wxPanel* Config::CreateExportPanel()
+{
+  wxPanel *panel = new wxPanel(m_notebook, -1);
+
+  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(4, 2, 5, 5);
+  wxFlexGridSizer* vsizer = new wxFlexGridSizer(17,1,5,5);
+
+  wxStaticText *dc = new wxStaticText(panel, -1, _("Documentclass for TeX export:"));
+  m_documentclass = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, wxDefaultSize.GetY()));
+  grid_sizer->Add(dc, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  grid_sizer->Add(m_documentclass, 0, wxALL, 5);
+
+  wxStaticText *tp = new wxStaticText(panel, -1, _("Additional lines for the TeX preamble:"));
+  m_texPreamble = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, 100), wxTE_MULTILINE | wxHSCROLL);
+  grid_sizer->Add(tp, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  grid_sizer->Add(m_texPreamble, 0, wxALL, 5);
+
+  vsizer->Add(grid_sizer, 1, wxEXPAND, 5);
+
+  m_AnimateLaTeX = new wxCheckBox(panel, -1, _("Export animations to TeX (Images only move if the PDF viewer supports this)"));
+  vsizer->Add(m_AnimateLaTeX, 0, wxALL, 5);
+
+  m_TeXExponentsAfterSubscript = new wxCheckBox(panel, -1, _("LaTeX: Place exponents after, instead above subscripts"));
+  vsizer->Add(m_TeXExponentsAfterSubscript, 0, wxALL, 5);
+
+  m_flowedTextRequested = new wxCheckBox(panel, -1, _("HTML/Text Cells: Export all linebreaks"));
+  vsizer->Add(m_flowedTextRequested, 0, wxALL, 5);
+
+  m_exportInput = new wxCheckBox(panel, -1, _("Include input cells in the export of a worksheet"));
+  vsizer->Add(m_exportInput, 0, wxALL, 5);
+
+  m_exportContainsWXMX = new wxCheckBox(panel, -1, _("Add the .wxmx file to the HTML export"));
+  vsizer->Add(m_exportContainsWXMX, 0, wxALL, 5);
+
+  m_exportWithMathJAX = new wxCheckBox(panel, -1, _("Use MathJAX in HTML export"));
+  vsizer->Add(m_exportWithMathJAX, 0, wxALL, 5);
+  
+  vsizer->AddGrowableRow(10);
+  panel->SetSizer(vsizer);
+  vsizer->Fit(panel);
+
+  return panel;
+}
+
 wxPanel* Config::CreateOptionsPanel()
 {
   wxPanel *panel = new wxPanel(m_notebook, -1);
 
-  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(3, 2, 5, 5);
+  wxFlexGridSizer* grid_sizer = new wxFlexGridSizer(4, 2, 5, 5);
   wxFlexGridSizer* vsizer = new wxFlexGridSizer(16,1,5,5);
 
   wxStaticText *lang = new wxStaticText(panel, -1, _("Language:"));
@@ -389,11 +474,6 @@ wxPanel* Config::CreateOptionsPanel()
   grid_sizer->Add(m_language, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   vsizer->Add(grid_sizer, 1, wxEXPAND, 5);
 
-  wxStaticText *tp = new wxStaticText(panel, -1, _("Additional lines for the TeX preamble:"));
-  m_texPreamble = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, 100), wxTE_MULTILINE | wxHSCROLL);
-  grid_sizer->Add(tp, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
-  grid_sizer->Add(m_texPreamble, 0, wxALL, 5);
-
   wxStaticText *as = new wxStaticText(panel, -1, _("Autosave interval (minutes, 0 means: off)"));
   m_autoSaveInterval = new wxSpinCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(230, -1), wxSP_ARROW_KEYS, 0, 30);
   grid_sizer->Add(as, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
@@ -411,16 +491,6 @@ wxPanel* Config::CreateOptionsPanel()
   m_uncomressedWXMX = new wxCheckBox(panel, -1, _("Optimize wxmx files for version control"));
   vsizer->Add(m_uncomressedWXMX, 0, wxALL, 5);
 
-  m_AnimateLaTeX = new wxCheckBox(panel, -1, _("Export animations to TeX (Images only move if the PDF viewer supports this)"));
-  vsizer->Add(m_AnimateLaTeX, 0, wxALL, 5);
-
-  m_TeXExponentsAfterSubscript = new wxCheckBox(panel, -1, _("LaTeX: Place exponents after, instead above subscripts"));
-  vsizer->Add(m_TeXExponentsAfterSubscript, 0, wxALL, 5);
-
-  m_flowedTextRequested = new wxCheckBox(panel, -1, _("HTML/Text Cells: Export all linebreaks"));
-  vsizer->Add(m_flowedTextRequested, 0, wxALL, 5);
-
-  
   m_saveUntitled = new wxCheckBox(panel, -1, _("Ask to save untitled documents"));
   vsizer->Add(m_saveUntitled, 0, wxALL, 5);
 
@@ -438,7 +508,7 @@ wxPanel* Config::CreateMaximaPanel()
 {
   wxPanel* panel = new wxPanel(m_notebook, -1);
 
-  wxFlexGridSizer* sizer = new wxFlexGridSizer(8, 2, 0, 0);  
+  wxFlexGridSizer* sizer = new wxFlexGridSizer(10, 2, 0, 0);  
 
   wxStaticText *mp = new wxStaticText(panel, -1, _("Maxima program:"));
   m_maximaProgram = new wxTextCtrl(panel, -1, wxEmptyString, wxDefaultPosition, wxSize(250, -1), wxTE_RICH);
@@ -475,6 +545,14 @@ wxPanel* Config::CreateMaximaPanel()
   sizer->Add(m_wxcd, 0, wxALL, 5);
   sizer->Add(10, 10);
   #endif
+
+  m_abortOnError = new wxCheckBox(panel, -1, _("Abort evaluation on error"));
+  sizer->Add(m_abortOnError,0,wxALL, 5);
+  sizer->Add(10,10);
+  
+  m_pollStdOut = new wxCheckBox(panel, -1, _("Debug: Watch maxima's stdout stream"));
+  sizer->Add(m_pollStdOut,0,wxALL, 5);
+  sizer->Add(10,10);
   
   panel->SetSizer(sizer);
   sizer->Fit(panel);
@@ -518,6 +596,7 @@ wxPanel* Config::CreateStylePanel()
       _("Output labels"),
       _("Highlight (dpart)"),
       _("Text cell"),
+      _("Subsubsection cell"),
       _("Subsection cell"),
       _("Section cell"),
       _("Title cell"),
@@ -527,9 +606,18 @@ wxPanel* Config::CreateStylePanel()
       _("Active cell bracket"),
       _("Cursor"),
       _("Selection"),
-      _("Outdated cells")
+      _("Text equal to selection"),
+      _("Outdated cells"),
+      _("Code highlighting: Variables"),
+      _("Code highlighting: Functions"),
+      _("Code highlighting: Comments"),
+      _("Code highlighting: Numbers"),
+      _("Code highlighting: Strings"),
+      _("Code highlighting: Operators"),
+      _("Code highlighting: End of line")
     };
-  m_styleFor = new wxListBox(panel, listbox_styleFor, wxDefaultPosition, wxSize(200, -1), 23, m_styleFor_choices, wxLB_SINGLE);
+
+  m_styleFor = new wxListBox(panel, listbox_styleFor, wxDefaultPosition, wxSize(250, -1), 31, m_styleFor_choices, wxLB_SINGLE);
   m_getStyleFont = new wxButton(panel, style_font_family, _("Choose font"), wxDefaultPosition, wxSize(150, -1));
 #ifndef __WXMSW__
   m_styleColor = new ColorPanel(this, panel, color_id, wxDefaultPosition, wxSize(150, 30), wxSUNKEN_BORDER | wxFULL_REPAINT_ON_RESIZE);
@@ -591,14 +679,18 @@ void Config::WriteSettings()
   int i = 0;
   wxString search = wxT("maxima-htmldir");
   wxArrayString out;
+  bool abortOnError = m_abortOnError->GetValue();
+  bool pollStdOut   = m_pollStdOut->GetValue();
   wxString maxima = m_maximaProgram->GetValue();
   wxConfig *config = (wxConfig *)wxConfig::Get();
+  config->Write(wxT("abortOnError"), m_abortOnError->GetValue());
+  config->Write(wxT("pollStdOut"), m_pollStdOut->GetValue());
   config->Write(wxT("maxima"), m_maximaProgram->GetValue());
   config->Write(wxT("parameters"), m_additionalParameters->GetValue());
   config->Write(wxT("fontSize"), m_fontSize);
   config->Write(wxT("mathFontsize"), m_mathFontSize);
   config->Write(wxT("matchParens"), m_matchParens->GetValue());
-  config->Write(wxT("showLong"), m_showLong->GetValue());
+  config->Write(wxT("showLength"), m_showLength->GetSelection());
   config->Write(wxT("fixedFontTC"), m_fixedFontInTC->GetValue());
   config->Write(wxT("changeAsterisk"), m_changeAsterisk->GetValue());
   config->Write(wxT("enterEvaluates"), m_enterEvaluates->GetValue());
@@ -606,6 +698,7 @@ void Config::WriteSettings()
   config->Write(wxT("openHCaret"), m_openHCaret->GetValue());
   config->Write(wxT("insertAns"), m_insertAns->GetValue());
   config->Write(wxT("undoLimit"), m_undoLimit->GetValue());
+  config->Write(wxT("bitmapScale"), m_bitmapScale->GetValue());
   config->Write(wxT("fixReorderedIndices"), m_fixReorderedIndices->GetValue());
   config->Write(wxT("defaultPort"), m_defaultPort->GetValue());
   #ifdef __WXMSW__
@@ -621,10 +714,14 @@ void Config::WriteSettings()
   config->Write(wxT("AnimateLaTeX"), m_AnimateLaTeX->GetValue());
   config->Write(wxT("TeXExponentsAfterSubscript"), m_TeXExponentsAfterSubscript->GetValue());
   config->Write(wxT("flowedTextRequested"), m_flowedTextRequested->GetValue());
+  config->Write(wxT("exportInput"), m_exportInput->GetValue());
+  config->Write(wxT("exportContainsWXMX"), m_exportContainsWXMX->GetValue());
+  config->Write(wxT("exportWithMathJAX"), m_exportWithMathJAX->GetValue());
   config->Write(wxT("usejsmath"), m_useJSMath->GetValue());
   config->Write(wxT("keepPercent"), m_keepPercentWithSpecials->GetValue());
   config->Write(wxT("texPreamble"), m_texPreamble->GetValue());
   config->Write(wxT("autoSaveInterval"), m_autoSaveInterval->GetValue());
+  config->Write(wxT("documentclass"), m_documentclass->GetValue());
   if (m_saveSize->GetValue())
     config->Write(wxT("pos-restore"), 1);
   else
@@ -689,7 +786,7 @@ void Config::OnChangeFontFamily(wxCommandEvent& event)
   style *tmp = GetStylePointer();
   wxString fontName;
 
-  if (tmp == &m_styleText || tmp == &m_styleTitle || tmp == &m_styleSubsection || tmp == &m_styleSection)
+  if (tmp == &m_styleText || tmp == &m_styleTitle || tmp == &m_styleSubsubsection || tmp == &m_styleSubsection || tmp == &m_styleSection)
   {
     if (tmp->fontSize != 0)
       fontsize = tmp->fontSize;
@@ -757,7 +854,7 @@ void Config::ReadStyles(wxString file)
                    &tmp)) m_styleBackground.color.Set(tmp);
 
   // Text background
-  m_styleTextBackground.color = wxT("light blue");
+  m_styleTextBackground.color = wxT("white");
   if (config->Read(wxT("Style/TextBackground/color"),
                    &tmp)) m_styleTextBackground.color.Set(tmp);
 
@@ -795,6 +892,11 @@ void Config::ReadStyles(wxString file)
   if (config->Read(wxT("Style/Selection/color"),
                    &tmp)) m_styleSelection.color.Set(tmp);
 
+  // Text equal to the current selection
+  m_styleEqualsSelection.color = wxT("rgb(192,192,255)");
+  if (config->Read(wxT("Style/EqualsSelection/color"),
+                   &tmp)) m_styleEqualsSelection.color.Set(tmp);
+
 #define READ_STYLE(style, where)                                        \
   if (config->Read(wxT(where "color"), &tmp)) style.color.Set(tmp);     \
   config->Read(wxT(where "bold"),                                       \
@@ -860,6 +962,13 @@ void Config::ReadStyles(wxString file)
   m_styleString.underlined = false;
   READ_STYLE(m_styleString, "Style/String/")
 
+  // Operator
+  m_styleString.color = m_styleDefault.color;
+  m_styleString.bold = false;
+  m_styleString.italic = true;
+  m_styleString.underlined = false;
+  READ_STYLE(m_styleString, "Style/String/Operator")
+
   // Greek
   m_styleGreek.color = m_styleDefault.color;
   m_styleGreek.bold = false;
@@ -883,7 +992,7 @@ void Config::ReadStyles(wxString file)
 
   // Text
   m_styleText.color = wxT("black");
-  m_styleText.bold = false;
+  m_styleText.bold = true;
   m_styleText.italic = false;
   m_styleText.underlined = false;
   m_styleText.font = m_styleDefault.font;
@@ -894,6 +1003,66 @@ void Config::ReadStyles(wxString file)
                &m_styleText.font);
   READ_STYLE(m_styleText, "Style/Text/")
 
+  // Variables in highlighted code
+  m_styleCodeHighlightingVariable.color = wxT("rgb(0,128,0)");
+  m_styleCodeHighlightingVariable.bold = false;
+  m_styleCodeHighlightingVariable.italic = true;
+  m_styleCodeHighlightingVariable.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingVariable, "Style/CodeHighlighting/Variable/")
+
+  // Functions in highlighted code
+  m_styleCodeHighlightingFunction.color = wxT("rgb(128,0,0)");
+  m_styleCodeHighlightingFunction.bold = false;
+  m_styleCodeHighlightingFunction.italic = true;
+  m_styleCodeHighlightingFunction.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingFunction, "Style/CodeHighlighting/Function/")
+
+  // Comments in highlighted code
+  m_styleCodeHighlightingComment.color = wxT("rgb(64,64,64)");
+  m_styleCodeHighlightingComment.bold = false;
+  m_styleCodeHighlightingComment.italic = true;
+  m_styleCodeHighlightingComment.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingComment, "Style/CodeHighlighting/Comment/")
+
+  // Numbers in highlighted code
+  m_styleCodeHighlightingNumber.color = wxT("rgb(128,64,0)");
+  m_styleCodeHighlightingNumber.bold = false;
+  m_styleCodeHighlightingNumber.italic = true;
+  m_styleCodeHighlightingNumber.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingNumber, "Style/CodeHighlighting/Number/")
+
+  // Strings in highlighted code
+  m_styleCodeHighlightingString.color = wxT("rgb(0,0,128)");
+  m_styleCodeHighlightingString.bold = false;
+  m_styleCodeHighlightingString.italic = true;
+  m_styleCodeHighlightingString.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingString, "Style/CodeHighlighting/String/")
+
+  // Operators in highlighted code
+  m_styleCodeHighlightingOperator.color = wxT("rgb(0,0,128)");
+  m_styleCodeHighlightingOperator.bold = false;
+  m_styleCodeHighlightingOperator.italic = true;
+  m_styleCodeHighlightingOperator.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingOperator, "Style/CodeHighlighting/Operator/")
+  // Line endings in highlighted code
+  m_styleCodeHighlightingEndOfLine.color = wxT("rgb(192,192,192)");
+  m_styleCodeHighlightingEndOfLine.bold = false;
+  m_styleCodeHighlightingEndOfLine.italic = true;
+  m_styleCodeHighlightingEndOfLine.underlined = false;
+  READ_STYLE(m_styleCodeHighlightingEndOfLine, "Style/CodeHighlighting/EndOfLine/")
+
+  // Subsubsection
+  m_styleSubsubsection.color = wxT("black");
+  m_styleSubsubsection.bold = true;
+  m_styleSubsubsection.italic = false;
+  m_styleSubsubsection.underlined = false;
+  m_styleSubsubsection.font = m_styleDefault.font;
+  m_styleSubsubsection.fontSize = 16;
+  config->Read(wxT("Style/Subsubsection/fontsize"),
+               &m_styleSubsubsection.fontSize);
+  config->Read(wxT("Style/Subsubsection/fontname"),
+               &m_styleSubsubsection.font);
+  READ_STYLE(m_styleSubsubsection, "Style/Subsubsection/")
   // Subsection
   m_styleSubsection.color = wxT("black");
   m_styleSubsection.bold = true;
@@ -970,6 +1139,8 @@ void Config::WriteStyles(wxString file)
                 m_styleCursor.color.GetAsString(wxC2S_CSS_SYNTAX));
   config->Write(wxT("Style/Selection/color"),
                 m_styleSelection.color.GetAsString(wxC2S_CSS_SYNTAX));
+  config->Write(wxT("Style/EqualsSelection/color"),
+                m_styleEqualsSelection.color.GetAsString(wxC2S_CSS_SYNTAX));
   config->Write(wxT("Style/Outdated/color"),
                 m_styleOutdated.color.GetAsString(wxC2S_CSS_SYNTAX));
 
@@ -1019,7 +1190,21 @@ void Config::WriteStyles(wxString file)
   config->Write(wxT("Style/Text/fontsize"), m_styleText.fontSize);
   WRITE_STYLE(m_styleText, "Style/Text/")
 
-  // Section
+  // Syntax highlighting
+  WRITE_STYLE(m_styleCodeHighlightingVariable, "Style/CodeHighlighting/Variable/")
+  WRITE_STYLE(m_styleCodeHighlightingFunction, "Style/CodeHighlighting/Function/")
+  WRITE_STYLE(m_styleCodeHighlightingComment,  "Style/CodeHighlighting/Comment/")
+  WRITE_STYLE(m_styleCodeHighlightingNumber,   "Style/CodeHighlighting/Number/")
+  WRITE_STYLE(m_styleCodeHighlightingString,   "Style/CodeHighlighting/String/")
+  WRITE_STYLE(m_styleCodeHighlightingOperator, "Style/CodeHighlighting/Operator/")
+  WRITE_STYLE(m_styleCodeHighlightingEndOfLine,"Style/CodeHighlighting/EndOfLine/")
+
+  // Subsubsection
+  config->Write(wxT("Style/Subsubsection/fontname"), m_styleSubsubsection.font);
+  config->Write(wxT("Style/Subsubsection/fontsize"), m_styleSubsubsection.fontSize);
+  WRITE_STYLE(m_styleSubsubsection, "Style/Subsubsection/")
+
+  // Subsection
   config->Write(wxT("Style/Subsection/fontname"), m_styleSubsection.font);
   config->Write(wxT("Style/Subsection/fontsize"), m_styleSubsection.fontSize);
   WRITE_STYLE(m_styleSubsection, "Style/Subsection/")
@@ -1067,13 +1252,13 @@ void Config::OnChangeStyle(wxCommandEvent& event)
 
   m_styleColor->SetBackgroundColour(tmp->color);
 
-  if (st >= 12 && st <= 15)
+  if (st >= TS_TEXT_BACKGROUND && st <= TS_SUBSUBSECTION)
     m_getStyleFont->Enable(true);
   else
     m_getStyleFont->Enable(false);
 
   // Background color only
-  if (st >= 16)
+  if (st >= TS_SECTION)
   {
     m_boldCB->Enable(false);
     m_italicCB->Enable(false);
@@ -1081,12 +1266,20 @@ void Config::OnChangeStyle(wxCommandEvent& event)
   }
   else
   {
-    m_boldCB->Enable(true);
-    m_italicCB->Enable(true);
-    m_underlinedCB->Enable(true);
-    m_boldCB->SetValue(tmp->bold);
-    m_italicCB->SetValue(tmp->italic);
-    m_underlinedCB->SetValue(tmp->underlined);
+    if(st>TS_OUTDATED)
+    {
+      m_boldCB->Enable(false);
+      m_italicCB->Enable(false);
+      m_underlinedCB->Enable(false);
+    } else
+    {
+      m_boldCB->Enable(true);
+      m_italicCB->Enable(true);
+      m_underlinedCB->Enable(true);
+      m_boldCB->SetValue(tmp->bold);
+      m_italicCB->SetValue(tmp->italic);
+      m_underlinedCB->SetValue(tmp->underlined);
+    }
   }
   UpdateExample();
 }
@@ -1111,7 +1304,7 @@ void Config::OnChangeWarning(wxCommandEvent &event)
 
 style* Config::GetStylePointer()
 {
-  style* tmp = &m_styleDefault;
+  style* tmp;
   switch (m_styleFor->GetSelection())
   {
   case 1:
@@ -1151,35 +1344,64 @@ style* Config::GetStylePointer()
     tmp = &m_styleText;
     break;
   case 13:
-    tmp = &m_styleSubsection;
+    tmp = &m_styleSubsubsection;
     break;
   case 14:
-    tmp = &m_styleSection;
+    tmp = &m_styleSubsection;
     break;
   case 15:
-    tmp = &m_styleTitle;
+    tmp = &m_styleSection;
     break;
   case 16:
-    tmp = &m_styleTextBackground;
+    tmp = &m_styleTitle;
     break;
   case 17:
-    tmp = &m_styleBackground;
+    tmp = &m_styleTextBackground;
     break;
   case 18:
-    tmp = &m_styleCellBracket;
+    tmp = &m_styleBackground;
     break;
   case 19:
-    tmp = &m_styleActiveCellBracket;
+    tmp = &m_styleCellBracket;
     break;
   case 20:
-    tmp = &m_styleCursor;
+    tmp = &m_styleActiveCellBracket;
     break;
   case 21:
-    tmp = &m_styleSelection;
+    tmp = &m_styleCursor;
     break;
   case 22:
+    tmp = &m_styleSelection;
+    break;
+  case 23:
+    tmp = &m_styleEqualsSelection;
+    break;
+  case 24:
     tmp = &m_styleOutdated;
     break;
+  case 25:
+    tmp = &m_styleCodeHighlightingVariable;
+    break;
+  case 26:
+    tmp = &m_styleCodeHighlightingFunction;
+    break;
+  case 27:
+    tmp = &m_styleCodeHighlightingComment;
+    break;
+  case 28:
+    tmp = &m_styleCodeHighlightingNumber;
+    break;
+  case 29:
+    tmp = &m_styleCodeHighlightingString;
+    break;
+  case 30:
+    tmp = &m_styleCodeHighlightingOperator;
+    break;
+  case 31:
+    tmp = &m_styleCodeHighlightingEndOfLine;
+    break;
+  default:
+    tmp = &m_styleDefault;
   }
   return tmp;
 }
@@ -1195,7 +1417,7 @@ void Config::UpdateExample()
     color = m_styleInput.color;
 
   int fontsize = m_fontSize;
-  if (tmp == &m_styleText || tmp == &m_styleSubsection || tmp == &m_styleSection || tmp == &m_styleTitle)
+  if (tmp == &m_styleText || tmp == &m_styleSubsubsection || tmp == &m_styleSubsection || tmp == &m_styleSection || tmp == &m_styleTitle)
   {
     fontsize = tmp->fontSize;
     font = tmp->font;
