@@ -148,8 +148,10 @@ private:
   void SetProperties();
   //! The panel that allows to set the editing options
   wxPanel* CreateWorksheetPanel();
-  //! The panel that allows to set checkbox-style options
+  //! A panel that allows to set general options
   wxPanel* CreateOptionsPanel();
+  //! The panel that allows to set options affecting the export functionality
+  wxPanel* CreateExportPanel();
   //! The panel that allows to change styles
   wxPanel* CreateStylePanel();
   //! The panel that allows to change maxima-specific configurations.
@@ -158,12 +160,15 @@ private:
 protected:
   // begin wxGlade: Config::attributes
   wxTextCtrl* m_maximaProgram;
+  wxTextCtrl* m_documentclass;
   wxTextCtrl* m_texPreamble;
   wxSpinCtrl* m_autoSaveInterval;
   wxButton* m_mpBrowse;
   wxTextCtrl* m_additionalParameters;
   wxComboBox* m_language;
   wxCheckBox* m_saveSize;
+  wxCheckBox* m_abortOnError;
+  wxCheckBox* m_pollStdOut;
   wxCheckBox* m_savePanes;
   wxCheckBox* m_usepngCairo;
   wxCheckBox* m_uncomressedWXMX;
@@ -177,13 +182,19 @@ protected:
   wxCheckBox* m_TeXExponentsAfterSubscript;
   //! A checkbox that asks if all newlines in text cells have to be passed to HTML.
   wxCheckBox* m_flowedTextRequested;
+  //! A checkbox that asks if we want to export the input for maxima, as well.
+  wxCheckBox* m_exportInput;
+  wxCheckBox* m_exportContainsWXMX;
+  wxCheckBox* m_exportWithMathJAX;
   wxCheckBox* m_matchParens;
-  wxCheckBox* m_showLong;
+
+  wxChoice* m_showLength;
   wxCheckBox* m_enterEvaluates;
   wxCheckBox* m_saveUntitled;
   wxCheckBox* m_openHCaret;
   wxCheckBox* m_insertAns;
   wxSpinCtrl* m_undoLimit;
+  wxSpinCtrl* m_bitmapScale;
   wxCheckBox* m_fixReorderedIndices;
   wxButton* m_getFont;
   wxButton* m_getStyleFont;
@@ -214,28 +225,37 @@ protected:
   ExamplePanel* m_examplePanel;
   // end wxGlade
   style m_styleDefault,
-        m_styleVariable,
-        m_styleFunction,
-        m_styleNumber,
-        m_styleSpecial,
-        m_styleGreek,
-        m_styleString,
-        m_styleInput,
-        m_styleMainPrompt,
-        m_styleOtherPrompt,
-        m_styleLabel,
-        m_styleHighlight,
-        m_styleText,
-        m_styleSubsection,
-        m_styleSection,
-        m_styleTitle,
-        m_styleTextBackground,
-        m_styleBackground,
-        m_styleCellBracket,
-        m_styleActiveCellBracket,
-        m_styleCursor,
-        m_styleSelection,
-        m_styleOutdated;
+    m_styleVariable,
+    m_styleFunction,
+    m_styleNumber,
+    m_styleSpecial,
+    m_styleGreek,
+    m_styleString,
+    m_styleInput,
+    m_styleMainPrompt,
+    m_styleOtherPrompt,
+    m_styleLabel,
+    m_styleHighlight,
+    m_styleText,
+    m_styleSubsubsection,
+    m_styleSubsection,
+    m_styleSection,
+    m_styleTitle,
+    m_styleTextBackground,
+    m_styleBackground,
+    m_styleCellBracket,
+    m_styleActiveCellBracket,
+    m_styleCursor,
+    m_styleSelection,
+    m_styleEqualsSelection,
+    m_styleOutdated,
+    m_styleCodeHighlightingVariable,
+    m_styleCodeHighlightingFunction,
+    m_styleCodeHighlightingComment,
+    m_styleCodeHighlightingNumber,
+    m_styleCodeHighlightingString,
+    m_styleCodeHighlightingOperator,
+    m_styleCodeHighlightingEndOfLine;
 
   //! Is called when the configuration dialog is closed.
   void OnClose(wxCloseEvent& event);
