@@ -26,6 +26,9 @@
 #include "MathCell.h"
 
 #include <vector>
+#include <list>
+#include <wx/tokenzr.h>
+
 /*! \file
 
   This file contains the definition of the class EditorCell
@@ -39,13 +42,57 @@
  */
 class EditorCell : public MathCell
 {
+private:
+  //! Draw a box that marks the current selection
+  void MarkSelection(size_t start, size_t end,CellParser& parser,double scale, wxDC& dc, TextStyle style);
+  /*! The start of the current selection.
+
+     - >0: the position of the cursors in characters from start
+     - -1: Currently no selection is active
+   */
+  long m_selectionStart;
+  /*! The end of the current selection.
+
+     - >0: the position of the cursors in characters from start
+     - -1: Currently no selection is active
+   */
+  long m_selectionEnd;
+  /*! The currently selected string. 
+
+    Since this string is defined statically it is available in every editor cell
+    for highlighting selected strings.
+  */
+  static wxString m_selectionString;
+  long m_oldSelectionStart;
+  long m_oldSelectionEnd;
+
 public:
+  //! Has the selection changed since the last draw event?
+  bool m_selectionChanged;
   //! The constructor
-  EditorCell();
+  EditorCell(wxString text = wxEmptyString);
   //! The destructor
   ~EditorCell();
+
+  /*! Expand all tabulators.
+
+    \param input The string the tabulators should be expanded in
+    \param posInLine The number of characters that come before the input in the same line
+    \todo Implement the actual TAB expansion
+  */
+  wxString TabExpand(wxString input, size_t posInLine);
+  //! Escape all chars that cannot be used in HTML otherwise
+  static wxString EscapeHTMLChars(wxString input);
+  //! Convert all but the first of a row of multiple spaces to non-breakable
+  static wxString PrependNBSP(wxString input);
+
   void Destroy();
   MathCell* Copy();
+  /*! Recalculate the widths of the current cell.
+
+      \todo If we use a centered dot instead of a * and we don't use a fixed-
+      size fonts we miscalculate the widths here.
+   */
   void RecalculateWidths(CellParser& parser, int fontsize);
   void Draw(CellParser& parser, wxPoint point, int fontsize);
   //! Convert the current cell to a string
@@ -54,17 +101,31 @@ public:
   wxString ToTeX();
   //! Convert the current cell to XML code for inclusion in a .wxmx file.
   wxString ToXML();
+  //! Convert the current cell to HTML code.
+  wxString ToHTML();
   void SetFont(CellParser& parser, int fontsize);
   void SetForeground(CellParser& parser);
+
+  /*! Sets the text that is to be displayed.
+    
+    Automatically calls StyleText().
+   */
   void SetValue(wxString text);
   wxString GetValue()
   {
     return m_text;
   }
+  /*! Converts m_text to a list of styled text snippets that will later be used by draw().
+
+    \todo Actually set the needed text styles.
+   */
+  void StyleText();
   void Reset();
   //! Decide what to do if the user pressed a key when this cell was selected
   void ProcessEvent(wxKeyEvent& event);
   bool ActivateCell();
+  //! Return the index of the 1st char of the line containing the letter #pos.
+  size_t BeginningOfLine(size_t pos);
   bool AddEnding();
   void PositionToXY(int pos, int* line, int* col);
   int XYToPosition(int x, int y);
@@ -81,6 +142,15 @@ public:
     m_selectionStart = 0;
     m_selectionEnd = m_positionOfCaret = m_text.Length();
   }
+  void SelectNone()
+  {
+    m_selectionStart = m_selectionEnd = 0;
+  }
+  bool SelectionActive()
+  {
+    return (m_selectionStart != -1)&&(m_selectionEnd != -1);
+  }
+
   bool CanCopy()
   {
     return m_selectionStart != -1;
@@ -89,18 +159,20 @@ public:
   {
     m_matchParens = match;
   }
-  void SetInsertAns(bool insertAns)
+    void SetInsertAns(bool insertAns)
   {
     m_insertAns = insertAns;
   }
   bool FindMatchingQuotes();
   void FindMatchingParens();
-  wxString GetLineString(int line, int start = 0, int end = -1);
+  int GetLineWidth(wxDC& dc, int line, int end);
   //! true, if this cell's width has to be recalculated.
   bool IsDirty()
   {
     return m_isDirty;
   }
+
+  //! Toggles the visibility of the cursor which is used to make it blink.
   void SwitchCaretDisplay()
   {
     m_displayCaret = !m_displayCaret;
@@ -110,7 +182,10 @@ public:
     m_hasFocus = focus;
   }
   void SetFirstLineOnly(bool show = true) {
-    if (m_firstLineOnly != show) { m_width = m_height = -1; m_firstLineOnly = show; }}
+    if (m_firstLineOnly != show) { m_width = m_height = -1; m_firstLineOnly = show; }
+    // Style the text anew.
+    StyleText();
+  }
   bool IsActive() { return m_isActive; }
   //! Is the cursor at the start of this cell?
   bool CaretAtStart() { return m_positionOfCaret == 0; }
@@ -140,8 +215,11 @@ public:
   //! Set the information if this cell needs to be re-evaluated by maxima
   void ContainsChanges(bool changes) { m_containsChanges = m_containsChangesCheck = changes; }
   bool CheckChanges();
-  //! Replaces all occurrences of a given string
-  int ReplaceAll(wxString oldString, wxString newString);
+  /*! Replaces all occurrences of a given string
+
+    TODO: Implement the IgnoreCase case.
+   */
+  int ReplaceAll(wxString oldString, wxString newString,bool IgnoreCase);
   /*! Finds the next occurrences of a string
 
     \param str The string to search for
@@ -153,21 +231,23 @@ public:
      - false: Case-sensitive search
    */
   bool FindNext(wxString str, bool down, bool ignoreCase);
-  void SetSelection(int start, int end)
-  {
-    m_selectionStart = start;
-    m_positionOfCaret = m_selectionEnd = end;
-  }
+  void SetSelection(int start, int end);
   void GetSelection(int *start, int *end)
   {
     *start = m_selectionStart; *end = m_selectionEnd;
   }
   /*! Replace the current selection with a string
 
-    \todo This function sets the bool m_containsChanges to -1. I assume this should 
-    read false instead.
+    \param oldStr The old string in the selection. If this string doesn't match
+                  the selection this function doesn't replace it.
+    \param newString The new string oldStr has to be replaced by
+    \param keepSelected 
+      - true = we want the new string to be selected afterwards
+      - false = the selection is cleared after replacing the string
+        and moving the cursor to its end.
+    \param ignoreCase
    */
-  bool ReplaceSelection(wxString oldStr, wxString newString);
+  bool ReplaceSelection(wxString oldStr, wxString newString,bool keepSelected = false,bool ignoreCase = false);
   //! Convert the current selection to a string
   wxString GetSelectionString();
   //! Unselect everything
@@ -180,7 +260,73 @@ public:
     {
       return GetValue().Mid(1,m_selectionStart);
     }
+  //! Return to the selection after the cell has been left upwards
+  void ReturnToSelectionFromTop()
+    {
+      SetSelection(m_lastSelectionStart,0);
+    }
+  //! Return to the selection after the cell has been left downwards
+  void ReturnToSelectionFromBot()
+    {
+      SetSelection(m_lastSelectionStart,m_text.Length());
+    }
 private:
+  /*! Divide a string into tokens
+
+    Used when styling text.
+   */
+  wxArrayString StringToTokens(wxString string);
+
+  bool IsAlpha(wxChar c);
+  bool IsNum(wxChar c);
+  bool IsAlphaNum(wxChar c);
+  
+  /*! A piece of styled text for syntax highlighting
+
+   */
+  class StyledText
+  {
+  private:
+    //! The color of this text portion
+    TextStyle  m_style;
+    //! The text of this text portion
+    wxString m_text;
+    //! Do we really want to style this text portion different than the default?
+    bool m_styleThisText;
+  public:
+    //! Defines a piece of styled text
+    StyledText(TextStyle style,wxString text)
+      {
+        m_text = text;
+        m_style = style;
+        m_styleThisText = true;
+      }
+
+    //! Defines a piece of text with the default style
+    StyledText(wxString text)
+      {
+        m_text = text;
+        m_styleThisText = false;
+      }
+    //! Returns the piece of text
+    wxString GetText()
+      {
+        return m_text;
+      }
+    //! If StyleSet() is true this function returns the color of this text portion
+    TextStyle GetStyle()
+      {
+        return m_style;
+      }
+    // Has a individual text style been set for this text portion?
+    bool StyleSet()
+      {
+        return m_styleThisText;
+      }
+  };
+  
+  std::list<StyledText> m_styledText;
+
 #if wxUSE_UNICODE
   wxString InterpretEscapeString(wxString txt);
 #endif
@@ -193,8 +339,7 @@ private:
   //! Where inside this cell is the cursor?
   int m_positionOfCaret;
   int m_caretColumn;
-  long m_selectionStart;
-  long m_selectionEnd;
+  long m_lastSelectionStart;
 //  long m_oldStart, m_oldEnd;
   int m_numberOfLines;
   bool m_isActive;
