@@ -23,8 +23,6 @@
 
 #include <wx/config.h>
 #include <wx/clipbrd.h>
-#include <wx/tokenzr.h>
-
 #include "MarkDown.h"
 #include "GroupCell.h"
 #include "SlideShowCell.h"
@@ -98,6 +96,11 @@ GroupCell::GroupCell(int groupType, wxString initString) : MathCell()
       editor->SetType(MC_TYPE_SUBSECTION);
       AppendInput(editor);
       break;
+    case GC_TYPE_SUBSUBSECTION:
+      m_input->SetType(MC_TYPE_SUBSUBSECTION);
+      editor->SetType(MC_TYPE_SUBSUBSECTION);
+      AppendInput(editor);
+      break;
     case GC_TYPE_IMAGE:
       m_input->SetType(MC_TYPE_TEXT);
       editor->SetType(MC_TYPE_TEXT);
@@ -159,33 +162,59 @@ bool GroupCell::Empty()
 	  );
 }
 
-void GroupCell::DestroyOutput()
+void GroupCell::DestroyOutput(bool destroyFirst)
 {
   MathCell *tmp = m_output, *tmp1;
+
+  // If there isn't anything to do we can already return.
+  if(tmp == NULL)
+    return;
+
+  // If we are dealing with an image cell we don't delete the actual image.
+  if(!destroyFirst)
+  {
+    tmp1 = tmp;
+    tmp = tmp -> m_next;
+    tmp1->m_next = NULL;
+    tmp1->m_nextToDraw = NULL;
+  }
+
+  // Delete what is left of the output.
   while (tmp != NULL) {
     tmp1 = tmp;
     tmp = tmp->m_next;
     tmp1->Destroy();
     delete tmp1;
   }
-  m_output = NULL;
+  if(destroyFirst)
+  {
+    m_output = NULL;
+    m_lastInOutput = NULL;
+    m_appendedCells = NULL;
+  }
 }
 
-// when all=false (default) only reset input label of the current (code) cell
-// if all = true, then also reset next cells and folded cells
-void GroupCell::ResetInputLabel(bool all)
+void GroupCell::ResetInputLabel()
 {
   if (m_groupType == GC_TYPE_CODE) {
     if (m_input)
       m_input->SetValue(EMPTY_INPUT_LABEL);
   }
-  // if all, also reset input labels in the folded cells
-  else if (all && IsFoldable() && m_hiddenTree)
-    m_hiddenTree->ResetInputLabel(true);
+}
+
+void GroupCell::ResetInputLabelList()
+{
+  GroupCell *tmp=this;
+  while(tmp)
+  {
+    tmp->ResetInputLabel();
+      // also reset input labels in the folded cells
+    if (tmp->IsFoldable() && (tmp->m_hiddenTree))
+      tmp->m_hiddenTree->ResetInputLabelList();
+
+    tmp=dynamic_cast<GroupCell*>(tmp->m_next);
+  }
 
-  // reset the next cell
-  if (all && m_next)
-    dynamic_cast<GroupCell*>(m_next)->ResetInputLabel(true);
 }
 
 MathCell* GroupCell::Copy()
@@ -226,7 +255,7 @@ void GroupCell::SetInput(MathCell *input)
   if (m_input != NULL)
     delete m_input;
   m_input = input;
-  m_input->m_group = this;
+  m_input->SetParent(this);
 }
 
 void GroupCell::AppendInput(MathCell *cell)
@@ -258,7 +287,7 @@ void GroupCell::SetOutput(MathCell *output)
     DestroyOutput();
 
   m_output = output;
-  m_output->m_group = this;
+  m_output->SetParent(this);
 
   m_lastInOutput = m_output;
 
@@ -270,12 +299,14 @@ void GroupCell::SetOutput(MathCell *output)
 
 void GroupCell::RemoveOutput()
 {
-  DestroyOutput();
+  // If there is nothing to do we can skip the rest of this action.
+  if(m_output == NULL)
+    return;
+  
+  DestroyOutput(!(GetGroupType() == GC_TYPE_IMAGE));
   ResetSize();
-  m_height = GetEditable()->GetHeight();
-  m_output = NULL;
-  m_lastInOutput = NULL;
-  m_appendedCells = NULL;
+  if(GetGroupType() != GC_TYPE_IMAGE)
+    m_height = GetEditable()->GetHeight();
   m_hide = false;
 }
 
@@ -662,6 +693,11 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 {
   wxString str;
   bool SuppressLeadingNewlines = true;
+  // Now we might want to introduce some markdown:
+  MarkDownTeX MarkDownParser;
+
+  bool exportInput = true;
+  wxConfig::Get()->Read(wxT("exportInput"), &exportInput);
 
   // pagebreak
   if (m_groupType == GC_TYPE_PAGEBREAK) {
@@ -682,7 +718,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
     if (!wxDirExists(imgDir))
       wxMkdir(imgDir);
 
-    if (bmp.ToFile(file))
+    if (bmp.ToFile(file).x>=0)
     {
       str << wxT("\\begin{figure}[htb]\n")
           << wxT("  \\begin{center}\n")
@@ -699,26 +735,31 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
   // CODE CELLS
   else if (m_groupType == GC_TYPE_CODE) {
     // Input cells
-    str = wxT("\n\\noindent\n%%%%%%%%%%%%%%%\n")
-          wxT("%%% INPUT:\n")
-          wxT("\\begin{minipage}[t]{8ex}{\\color{red}\\bf\n")
-          wxT("\\begin{verbatim}\n") +
-          m_input->ToString() +
-          wxT("\n\\end{verbatim}}\n\\end{minipage}");
-
-    if (m_input->m_next!=NULL)
+    if(exportInput)
     {
-
-      wxString input = m_input->m_next->ToString();
+      str = wxT("\n\\noindent\n%%%%%%%%%%%%%%%\n")
+        wxT("%%% INPUT:\n")
+        wxT("\\begin{minipage}[t]{8ex}\\color{red}\\bf\n")
+        wxT("\\begin{verbatim}\n") +
+        m_input->ToString() +
+        wxT("\n\\end{verbatim}\n\\end{minipage}");
+      
+      if (m_input->m_next!=NULL)
+      {
+        
+        wxString input = m_input->m_next->ToString();
 #if wxUSE_UNICODE
-      input.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
+        input.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
+        input.Replace(wxT("\xDCB6"), wxT(" ")); // Some weird unicode space character
 #endif
-      str += wxT("\n\\begin{minipage}[t]{\\textwidth}{\\color{blue}\n\\begin{verbatim}\n") +
-             input +
-             wxT("\n\\end{verbatim}}\n\\end{minipage}");
+        str += wxT("\n\\begin{minipage}[t]{\\textwidth}\\color{blue}\n\\begin{verbatim}\n") +
+          input +
+             wxT("\n\\end{verbatim}\n\\end{minipage}");
+      }
+      
+      str += wxT("\n");
     }
-
-    str += wxT("\n");
+    else str = wxEmptyString;
 
     if (m_output != NULL) {
       str += wxT("%%% OUTPUT:\n");
@@ -768,7 +809,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
               
               Bitmap bmp;
               bmp.SetData(copy);
-              if (bmp.ToFile(file))
+              if (bmp.ToFile(file).x>=0)
                 str += wxT("\\includegraphics[width=.95\\linewidth,height=.80\\textheight,keepaspectratio]{") +
                   filename + wxT("_img/") + image + wxT("}");
               else
@@ -779,13 +820,13 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
         else if (tmp->GetStyle() == TS_LABEL)
         {
 	  if(mathMode)
-	    str += wxT("\\end{math}\n\n\\begin{math}\\displaystyle\n");
+	    str += wxT("\\]\n\\[\\displaystyle\n");
 	  else
 	    {
-	      str += wxT("\n\n\\begin{math}\\displaystyle\n");
+	      str += wxT("\n\\[\\displaystyle\n");
 	      mathMode=true;
 	    }
-          str += wxT("\\parbox{8ex}{\\color{labelcolor}") + tmp->ToTeX() + wxT("}\n");
+          str += wxT("\\parbox{10ex}{$\\color{labelcolor}") + tmp->ToTeX() + wxT("$}\n");
         }
 	
         else
@@ -797,7 +838,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 		if((mathMode)&&(tmp->ToString().Length()>2))
 		  {
 		    str += wxT("\\mbox{}");
-		    str += wxT("\n\\end{math}\n");
+		    str += wxT("\n\\]\n");
 		    mathMode = false;
 		  }
 	      }
@@ -805,7 +846,7 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 	      {
 		if(!mathMode)
 		  {
-		    str += wxT("\n\n\\begin{math}\\displaystyle\n");
+		    str += wxT("\n\n\\[\\displaystyle\n");
 		    mathMode = true;
 		  }
 	      }		
@@ -818,12 +859,12 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
 	  // Some invisible dummy content that keeps TeX happy if there really is
 	  // no output to display.
 	  str += wxT("\\mbox{}");
-	  str += wxT("\n\\end{math}\n%%%%%%%%%%%%%%%\n");
+	  str += wxT("\n\\]\n%%%%%%%%%%%%%%%\n");
 	}
     }
   }
 
-  // TITLES, SECTIONS, SUBSECTIONS, TEXT
+  // TITLES, SECTIONS, SUBSECTIONS, SUBSUBSECTIONS, TEXT
   else if (GetEditable() != NULL && !m_hide) {
     str = GetEditable()->ListToTeX();
     switch (GetEditable()->GetStyle()) {
@@ -838,14 +879,14 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
       case TS_SUBSECTION:
         str = wxT("\n\\subsection{") + PrepareForTeX(str) + wxT("}\n\n");
         break;
+      case TS_SUBSUBSECTION:
+        str = wxT("\n\\subsubsection{") + PrepareForTeX(str) + wxT("}\n\n");
+        break;
       default:
         if (str.StartsWith(wxT("TeX:")))
           str = str.Mid(5, str.Length());
         else {
           str = PrepareForTeX(str);
-
-	  // Now we might want to introduce some markdown:
-	  MarkDownTeX MarkDownParser;
 	  str = MarkDownParser.MarkDown(str);
         }
         break;
@@ -871,13 +912,19 @@ wxString GroupCell::ToXML()
       str += wxT(" type=\"text\"");
       break;
     case GC_TYPE_TITLE:
-      str += wxT(" type=\"title\"");
+      str += wxT(" type=\"title\" sectioning_level=\"1\"");
       break;
     case GC_TYPE_SECTION:
-      str += wxT(" type=\"section\"");
+      str += wxT(" type=\"section\" sectioning_level=\"2\"");
       break;
     case GC_TYPE_SUBSECTION:
-      str += wxT(" type=\"subsection\"");
+      str += wxT(" type=\"subsection\" sectioning_level=\"3\"");
+      break;
+    case GC_TYPE_SUBSUBSECTION:
+      // We save subsubsections as subsections with a higher sectioning level:
+      // This makes them backwards-compatible in the way that they are displayed
+      // as subsections on old wxMaxima installations.
+      str += wxT(" type=\"subsection\" sectioning_level=\"4\"");
       break;
     case GC_TYPE_PAGEBREAK:
       {
@@ -925,6 +972,7 @@ wxString GroupCell::ToXML()
     case GC_TYPE_TITLE:
     case GC_TYPE_SECTION:
     case GC_TYPE_SUBSECTION:
+    case GC_TYPE_SUBSUBSECTION:
       if (input)
         str += input->ListToXML();
       if (m_hiddenTree) {
@@ -1076,7 +1124,8 @@ EditorCell *GroupCell::GetEditable()
     case GC_TYPE_TITLE:
     case GC_TYPE_SECTION:
     case GC_TYPE_SUBSECTION:
-      return dynamic_cast<EditorCell*>(GetInput());
+    case GC_TYPE_SUBSUBSECTION:
+      return GetInput();
     case GC_TYPE_PAGEBREAK:
     default:
       return NULL;
@@ -1292,34 +1341,44 @@ GroupCell *GroupCell::Unfold() {
   return dynamic_cast<GroupCell*>(tmp);
 }
 
-GroupCell *GroupCell::FoldAll(bool all) {
-  GroupCell *result = this;
-  if (IsFoldable() && !m_hiddenTree) {
-    Fold();
-    if (m_hiddenTree)
-      m_hiddenTree->FoldAll(true);
-  }
-  else
-    result = NULL;
+GroupCell *GroupCell::FoldAll() {
+  GroupCell *result = NULL;
 
-  if (all && m_next)
-    return dynamic_cast<GroupCell*>(m_next)->FoldAll(true);
-  else
-    return result;
+  GroupCell *tmp = this;
+
+  while (tmp != NULL)
+  {
+    if (tmp->IsFoldable() && !tmp->m_hiddenTree)
+    {
+      tmp->Fold();
+      result = tmp;
+    }
+    if (tmp-> m_hiddenTree)
+      m_hiddenTree->FoldAll();
+    tmp = dynamic_cast<GroupCell*>(tmp->m_next);
+  }
+  return result;
 }
 
 // unfolds recursivly its contents
 // if (all) then also calls it on it's m_next
-GroupCell *GroupCell::UnfoldAll(bool all) {
-  if (all && m_next)
-    dynamic_cast<GroupCell*>(m_next)->UnfoldAll(true);
+GroupCell *GroupCell::UnfoldAll() {
+  GroupCell *result = NULL;
 
-  if (!IsFoldable() || !m_hiddenTree)
-    return NULL;
+  GroupCell *tmp = this;
 
-  m_hiddenTree->UnfoldAll(true);
-
-  return Unfold();
+  while (tmp != NULL)
+  {
+    if (tmp->IsFoldable() && tmp->m_hiddenTree)
+    {
+      tmp->Unfold();
+      result = tmp;
+    }
+    if (tmp-> m_hiddenTree)
+      m_hiddenTree->UnfoldAll();
+    tmp = dynamic_cast<GroupCell*>(tmp->m_next);
+  }
+  return result;
 }
 
 bool GroupCell::IsLesserGCType(int comparedTo) {
@@ -1328,8 +1387,14 @@ bool GroupCell::IsLesserGCType(int comparedTo) {
     case GC_TYPE_TEXT:
     case GC_TYPE_PAGEBREAK:
     case GC_TYPE_IMAGE:
-      if ((comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION) ||
-          (comparedTo == GC_TYPE_SUBSECTION))
+      if ((comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION)  ||
+          (comparedTo == GC_TYPE_SUBSECTION) || (comparedTo == GC_TYPE_SUBSUBSECTION))
+        return true;
+      else
+        return false;
+    case GC_TYPE_SUBSUBSECTION:
+      if ((comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION)
+        || (comparedTo == GC_TYPE_SUBSECTION))
         return true;
       else
         return false;
@@ -1350,14 +1415,14 @@ bool GroupCell::IsLesserGCType(int comparedTo) {
   }
 }
 
-void GroupCell::Number(int &section, int &subsection, int &image) {
+void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &image) {
   switch (m_groupType) {
     case GC_TYPE_TITLE:
-      section = subsection = 0;
+      section = subsection = subsubsection = 0;
       break;
     case GC_TYPE_SECTION:
       section++;
-      subsection = 0;
+      subsection = subsubsection = 0;
       {
         wxString num = wxT(" ");
         num << section << wxT(" ");
@@ -1365,6 +1430,7 @@ void GroupCell::Number(int &section, int &subsection, int &image) {
       }
       break;
     case GC_TYPE_SUBSECTION:
+      subsubsection = 0;
       subsection++;
       {
         wxString num = wxT("  ");
@@ -1372,6 +1438,14 @@ void GroupCell::Number(int &section, int &subsection, int &image) {
         ((TextCell*)m_input)->SetValue(num);
       }
       break;
+    case GC_TYPE_SUBSUBSECTION:
+      subsubsection++;
+      {
+        wxString num = wxT("  ");
+        num << section << wxT(".") << subsection << wxT(".") << subsubsection << wxT(" ");
+        ((TextCell*)m_input)->SetValue(num);
+      }
+      break;
     case GC_TYPE_IMAGE:
       image++;
       {
@@ -1384,10 +1458,10 @@ void GroupCell::Number(int &section, int &subsection, int &image) {
   }
 
   if (IsFoldable() && m_hiddenTree)
-    m_hiddenTree->Number(section, subsection, image);
+    m_hiddenTree->Number(section, subsection, subsubsection, image);
 
   if (m_next)
-    dynamic_cast<GroupCell*>(m_next)->Number(section, subsection, image);
+    dynamic_cast<GroupCell*>(m_next)->Number(section, subsection, subsubsection, image);
 }
 
 bool GroupCell::IsMainInput(MathCell *active)
