@@ -33,6 +33,7 @@ ImgCell::ImgCell() : MathCell()
   m_type = MC_TYPE_IMAGE;
   m_fileSystem = NULL;
   m_drawRectangle = true;
+  m_imageBorderWidth = 1;
 }
 
 int ImgCell::s_counter = 0;
@@ -149,29 +150,55 @@ void ImgCell::Destroy()
 
 void ImgCell::RecalculateWidths(CellParser& parser, int fontsize)
 {
+  int height;
   if (m_bitmap != NULL)
-    m_width = m_bitmap->GetWidth() + 2;
+  {
+    height = m_bitmap->GetHeight();
+    m_width  = m_bitmap->GetWidth();
+  }
   else
-    m_width = 0;
+  {
+    height = 0;
+    m_width  = 0;
+    }
 
   double scale = parser.GetScale();
   scale = MAX(scale, 1.0);
 
-  m_width = (int) (scale * m_width);
+  // Shrink to .9* the canvas size
+  if(scale * m_width > .9 * m_canvasSize.x)
+    scale = .9 * m_canvasSize.x / m_width;
+  if(scale * height > .9 * m_canvasSize.y)
+    scale = .9 * m_canvasSize.y / height;
+
+  m_width = (int) (scale * m_width) + 2 * m_imageBorderWidth;
   ResetData();
 }
 
 void ImgCell::RecalculateSize(CellParser& parser, int fontsize)
 {
+  int width;
   if (m_bitmap != NULL)
-    m_height = m_bitmap->GetHeight() + 2;
+  {
+    m_height = m_bitmap->GetHeight();
+    width  = m_bitmap->GetWidth();
+  }
   else
+  {
     m_height = 0;
-
+    width  = 0;
+  }
+  
   double scale = parser.GetScale();
   scale = MAX(scale, 1.0);
+  
+  // Shrink to .9* the canvas size
+  if(scale * width > .9 * m_canvasSize.x)
+    scale = .9 * m_canvasSize.x / width;
+  if(scale * m_height > .9 * m_canvasSize.y)
+    scale = .9 * m_canvasSize.y / m_height;
 
-  m_height= (int) (scale * m_height);
+  m_height= (int) (scale * m_height) + 2 * m_imageBorderWidth;
 
   m_center = m_height / 2;
 }
@@ -184,24 +211,30 @@ void ImgCell::Draw(CellParser& parser, wxPoint point, int fontsize)
   {
     wxMemoryDC bitmapDC;
     double scale = parser.GetScale();
-    scale = MAX(scale, 1.0);
 
     SetPen(parser);
     if (m_drawRectangle)
-      dc.DrawRectangle(wxRect(point.x, point.y - m_center, m_width, m_height));
+      
+      dc.DrawRectangle(wxRect(point.x, point.y - m_center, m_width, m_height));  
 
-    if (scale != 1.0)
+    bool rescale=false;
+    if(m_bitmap->GetHeight() + 2 * m_imageBorderWidth != m_height)
+      rescale=true;
+    
+    if (rescale)
     {
       wxImage img = m_bitmap->ConvertToImage();
-      img.Rescale(m_width, m_height);
+      img.Rescale(m_width - 2 * m_imageBorderWidth, m_height - 2 * m_imageBorderWidth,wxIMAGE_QUALITY_BICUBIC);
 
       wxBitmap bmp = img;
       bitmapDC.SelectObject(bmp);
     }
     else
+    {
       bitmapDC.SelectObject(*m_bitmap);
+    }
 
-    dc.Blit(point.x + 1, point.y - m_center + 1, m_width, m_height, &bitmapDC, 0, 0);
+    dc.Blit(point.x + m_imageBorderWidth, point.y - m_center + m_imageBorderWidth, m_width - 2 * m_imageBorderWidth, m_height - 2 * m_imageBorderWidth, &bitmapDC, 0, 0);
   }
 
   MathCell::Draw(parser, point, fontsize);
@@ -209,19 +242,36 @@ void ImgCell::Draw(CellParser& parser, wxPoint point, int fontsize)
 
 wxString ImgCell::ToString()
 {
-  return wxT(" << Graphics >> ");
+  return _(" (Graphics) ");
 }
 
 wxString ImgCell::ToTeX()
 {
-  return wxT(" << Graphics >> ");
+  return _(" (Graphics) ");
 }
 
-bool ImgCell::ToImageFile(wxString file)
+wxSize ImgCell::ToImageFile(wxString file)
 {
   wxImage image = m_bitmap->ConvertToImage();
 
-  return image.SaveFile(file, wxBITMAP_TYPE_PNG);
+  SizeInMillimeters retval;
+  if(image.SaveFile(file, wxBITMAP_TYPE_PNG))
+  {
+    // Saving an animation might need loads of time. Since we use this time
+    // in the foreground and many operation systems assume that an application
+    // that is busy with other things and therefore isn't reacting is stuck
+    // and therefore offer to kill the application we should now listen to
+    // requests from the OS before continuing the save.
+    wxYield();
+
+    return image.GetSize();
+  }
+  else
+  {
+    wxSize retval;
+    retval.x = retval.y = -1;
+    return retval;
+  }
 }
 
 wxString ImgCell::ToXML()
