@@ -45,6 +45,7 @@ MathCell::MathCell()
   m_type = MC_TYPE_DEFAULT;
   m_textStyle = TS_VARIABLE;
   m_SuppressMultiplicationDot = false;
+  m_imageBorderWidth = 0;
 }
 
 /***
@@ -77,6 +78,9 @@ void MathCell::SetType(int type)
   case MC_TYPE_TEXT:
     m_textStyle = TS_TEXT;
     break;
+  case MC_TYPE_SUBSUBSECTION:
+    m_textStyle = TS_SUBSUBSECTION;
+    break;
   case MC_TYPE_SUBSECTION:
     m_textStyle = TS_SUBSECTION;
     break;
@@ -155,26 +159,27 @@ void MathCell::AppendCell(MathCell *p_next)
 
 MathCell* MathCell::GetParent()
 {
+  wxASSERT_MSG(m_group != NULL,_("Bug: Math Cell that claims to have no group Cell it belongs to"));
   return m_group;
 }
 
 /***
  * Get the maximum drop of the center.
+
  */
 int MathCell::GetMaxCenter()
 {
-  if (m_maxCenter == -1)
+  if (m_maxCenter < 0)
   {
-    int center = m_isBroken ? 0 : m_center;
-    if (m_nextToDraw == NULL)
-      m_maxCenter = center;
-    else
+    MathCell *tmp = this;
+    while(tmp != NULL)
     {
-      // If the next cell is on a new line, maxCenter is m_center
-      if (m_nextToDraw->m_breakLine && !m_nextToDraw->m_isBroken)
-        m_maxCenter = center;
-      else
-        m_maxCenter = MAX(center, m_nextToDraw->GetMaxCenter());
+      m_maxCenter = MAX(m_maxCenter, tmp->m_center);
+      if(tmp->m_nextToDraw == NULL)
+        break;
+      if(tmp->m_nextToDraw->m_breakLine)
+        break;
+      tmp = tmp-> m_nextToDraw;
     }
   }
   return m_maxCenter;
@@ -182,20 +187,23 @@ int MathCell::GetMaxCenter()
 
 /***
  * Get the maximum drop of cell.
+
+\todo Convert this function to not using recursive function calls any more.
  */
 int MathCell::GetMaxDrop()
 {
-  if (m_maxDrop == -1)
+
+  if (m_maxDrop < 0)
   {
-    int drop = m_isBroken ? 0 : (m_height - m_center);
-    if (m_nextToDraw == NULL)
-      m_maxDrop = drop;
-    else
+    MathCell *tmp = this;
+    while(tmp != NULL)
     {
-      if (m_nextToDraw->m_breakLine && !m_nextToDraw->m_isBroken)
-        m_maxDrop = drop;
-      else
-        m_maxDrop = MAX(drop, m_nextToDraw->GetMaxDrop());
+      m_maxDrop = MAX(m_maxDrop, tmp->m_isBroken ? 0 : (tmp->m_height - tmp->m_center));
+      if(tmp->m_nextToDraw == NULL)
+        break;
+      if(tmp->m_nextToDraw->m_breakLine && !tmp->m_nextToDraw->m_isBroken)
+        break;
+      tmp = tmp-> m_nextToDraw;
     }
   }
   return m_maxDrop;
@@ -213,18 +221,27 @@ int MathCell::GetMaxHeight()
  */
 int MathCell::GetFullWidth(double scale)
 {
-  if (m_fullWidth == -1)
+  // Recalculate the with of this list of cells only if this has been marked as necessary.
+  if (m_fullWidth < 0)
   {
-    if (m_next == NULL)
-      m_fullWidth = m_width;
-    else
-      m_fullWidth = m_width + m_next->GetFullWidth(scale) +
-                    SCALE_PX(MC_CELL_SKIP, scale);
+    MathCell *tmp = this;
+
+    // We begin this calculation with a negative offset since the full width of only a single
+    // cell doesn't contain the space that separates two cells - that is automatically added
+    // to every cell in the next step.
+    m_fullWidth = -SCALE_PX(MC_CELL_SKIP, scale);
+    while(tmp != NULL)
+    {
+      m_fullWidth += tmp->m_width;
+      tmp = tmp->m_next + SCALE_PX(MC_CELL_SKIP, scale);
+    }
   }
   return m_fullWidth;
 }
 
 /*! Get the width of this line.
+
+\todo Convert this function to not using recursive function calls any more.
  */
 int MathCell::GetLineWidth(double scale)
 {
@@ -256,12 +273,12 @@ void MathCell::DrawList(CellParser& parser, wxPoint point, int fontsize)
 {
   MathCell *tmp=this;
   while(tmp!=NULL)
-    {
-      tmp->Draw(parser,point,fontsize);
-      double scale = parser.GetScale();
-      point.x += tmp->m_width + SCALE_PX(MC_CELL_SKIP, scale);
-      tmp=tmp->m_nextToDraw;
-    }
+  {
+    tmp->Draw(parser,point,fontsize);
+    double scale = parser.GetScale();
+    point.x += tmp->m_width + SCALE_PX(MC_CELL_SKIP, scale);
+    tmp=tmp->m_nextToDraw;
+  }
 }
 
 void MathCell::RecalculateSizeList(CellParser& parser, int fontsize)
@@ -371,11 +388,16 @@ wxString MathCell::ToString()
 wxString MathCell::ListToString()
 {
   wxString retval;
-  MathCell *tmp=this;
+  MathCell *tmp = this;
+  bool firstline = true;
   
   while(tmp!=NULL)
     {
       retval+=tmp->ToString();
+      if((!firstline)&&(tmp->m_forceBreakLine))
+        retval+=wxT("\n");
+
+      firstline = false;
       tmp=tmp->m_next;
     }
   
@@ -393,10 +415,13 @@ wxString MathCell::ListToTeX()
   MathCell *tmp=this;
   
   while(tmp!=NULL)
-    {
-      retval+=tmp->ToTeX();
-      tmp=tmp->m_next;
-    }
+  {
+    if ((tmp->m_textStyle == TS_LABEL && retval != wxEmptyString) ||
+        (tmp->m_breakLine && retval != wxEmptyString))
+      retval += wxT("\\]\\[");
+    retval += tmp->ToTeX();
+    tmp = tmp->m_next;
+  }
   
   return retval;
 }
@@ -414,27 +439,27 @@ wxString MathCell::ListToXML()
   MathCell *tmp=this;
 
   while(tmp!=NULL)
+  {
+    if((tmp->GetHighlight())&&(!highlight))
     {
-      if((tmp->GetHighlight())&&(!highlight))
-	{
-	  retval+=wxT("<hl>\n");
-	  highlight=true;
-	}
-
-      if((!tmp->GetHighlight())&&(highlight))
-	{
-	  retval+=wxT("</hl>\n");
-	  highlight=false;
-	}
-      
-      retval+=tmp->ToXML();
-      tmp=tmp->m_next;
+      retval+=wxT("<hl>\n");
+      highlight=true;
     }
-
-  if(highlight)
+    
+    if((!tmp->GetHighlight())&&(highlight))
     {
       retval+=wxT("</hl>\n");
+      highlight=false;
     }
+    
+    retval+=tmp->ToXML();
+    tmp=tmp->m_next;
+  }
+  
+  if(highlight)
+  {
+    retval+=wxT("</hl>\n");
+  }
   
   return retval;
 }
@@ -532,6 +557,25 @@ void MathCell::ResetData()
   m_breakLine = m_forceBreakLine;
 }
 
+
+MathCell *MathCell::first()
+{
+  MathCell *tmp=this;
+  while(tmp->m_previous)
+    tmp = tmp->m_previous;
+
+  return tmp;
+}
+
+MathCell *MathCell::last()
+{
+  MathCell *tmp=this;
+  while(tmp->m_next)
+    tmp = tmp->m_next;
+
+  return tmp;
+}
+
 void MathCell::Unbreak()
 {
   ResetData();
@@ -551,6 +595,17 @@ void MathCell::UnbreakList()
   }
 }
 
+void MathCell::DestroyList()
+{
+  MathCell *tmp, *next;
+  tmp = this;
+  while(tmp != NULL)
+  {
+    next = tmp->m_next;
+    tmp->Destroy();
+    tmp = next;
+  }
+}
 /***
  * Set the pen in device context accordint to the style of the cell.
  */
@@ -631,3 +686,5 @@ bool MathCell::IsMath()
            m_textStyle == TS_LABEL ||
            m_textStyle == TS_INPUT);
 }
+
+wxSize MathCell::m_canvasSize;
