@@ -33,6 +33,7 @@
 #include "GroupCell.h"
 #include "EvaluationQueue.h"
 #include "Autocomplete.h"
+#include "AutocompletePopup.h"
 #include "Structure.h"
 #include "ToolBar.h"
 
@@ -42,8 +43,7 @@ This canvas contains all the math, title, image etc.- cells of the current sessi
  */
 class MathCtrl: public wxScrolledCanvas
 {
-   private:
-
+   private:  
   /*! \defgroup UndoBufferFill
 
     These methods and classes contain the undo functionality for tree changes:
@@ -117,7 +117,8 @@ class MathCtrl: public wxScrolledCanvas
 
   //! The current undo action.
   TreeUndoAction m_currentUndoAction;
-  
+
+  //! Clear the list of actions for which an undo can be undone
   void TreeUndo_ClearRedoActionList();
 
   //! Remove one action ftom the action list
@@ -154,7 +155,7 @@ class MathCtrl: public wxScrolledCanvas
   /*! The cursor has entered one cell => save the value to see if it has changed.
   */
   void TreeUndo_CellEntered();
-  
+
   /*! The cursor is about to leave the current cell => Store the change if the value has changed.
    */
   void TreeUndo_CellLeft();
@@ -208,7 +209,6 @@ class MathCtrl: public wxScrolledCanvas
   //! true if m_start of m_currentUndoAction already marks the beginning of the action.
   bool m_TreeUndoMergeStartIsSet;
   //!@}
-
   
   bool m_scrolledAwayFromEvaluation;
 
@@ -219,6 +219,18 @@ class MathCtrl: public wxScrolledCanvas
     Also converts \n to <BR>
    */
   wxString EscapeHTMLChars(wxString input);
+
+  /*! Update the table of contents
+
+    This function actually only schedules the update of the table-of-contents-tab.
+    The actual update is done when wxMaxima is idle.
+   */
+  void UpdateTableOfContents()
+    {
+      m_scheduleUpdateToc = true;
+    }
+  
+  
   //! Allow indentation by spaces for html by replacing them by non-breakable spaces
   wxString PrependNBSP(wxString input);
   //! An enum for all classes of items one can click on
@@ -239,14 +251,35 @@ class MathCtrl: public wxScrolledCanvas
 
   //! Add a line to a file.
   void AddLineToFile(wxTextFile& output, wxString s, bool unicode = true);
+  //! Copy the currently selected cells
   MathCell* CopySelection();
+  /*! Copy the currently given list of cells
+
+    \param start The cell to start copying at
+    \param end   The cell the copy has to end with
+    \param asdata 
+      - true:  The cells are copied in the order they are stored
+      - false: The cells are copied in the order they are displayed: Sometimes (for 
+               example with fractions that can be displayed in 2d or flat) these
+               two orders differ.
+   */
   MathCell* CopySelection(MathCell* start, MathCell* end, bool asData = false);
   
   void GetMaxPoint(int* width, int* height);
+  //! Is executed if a timer associated with MathCtrl has expired.
   void OnTimer(wxTimerEvent& event);
+  /*! Has the autosave interval expired?
+  
+    True means: A save will be issued after the user stops typing.
+   */
   bool m_autoSaveIntervalExpired;
   void OnMouseExit(wxMouseEvent& event);
   void OnMouseEnter(wxMouseEvent& event);
+  /*! Is called by wxWidgets when it wants to redraw the console.
+
+    The canonical way to trigger this function is calling the Refresh() function
+    of this class.
+   */
   void OnPaint(wxPaintEvent& event);
   void OnSize(wxSizeEvent& event);
   void OnMouseRightDown(wxMouseEvent& event);
@@ -256,6 +289,7 @@ class MathCtrl: public wxScrolledCanvas
   void OnMouseLeftInGcLeft(wxMouseEvent& event, GroupCell *clickedInGC);
   void OnMouseLeftInGc(wxMouseEvent& event, GroupCell *clickedInGC);
   void OnMouseMotion(wxMouseEvent& event);
+  void OnMouseWheel(wxMouseEvent& event);
   void OnDoubleClick(wxMouseEvent& event);
   /*! A special key has been pressed 
 
@@ -283,13 +317,30 @@ class MathCtrl: public wxScrolledCanvas
   - keycode (ccode) is WXK_UP/WXK_DOWN
  */
   void SelectWithChar(int ccode);
+  /*!
+   * Select the rectangle surrounded by down and up. Called from OnMouseMotion.
+   *
+   * The method decides what to do, based on the value of m_clickType which
+   * was set previously in OnMouseLeftDown. This enables different selection behaviours
+   * depending on where we first clicked. If m_clickType equals
+   * CLICK_TYPE_NONE - click-dragging does not result in a selection (we clicked in hideRect for instance)
+   * CLICK_TYPE_GROUP_SELECTION - we are selecting full groupcells only. Only y-coordinate matters.
+   * CLICK_TYPE_INPUT_SELECTION - we clicked in an editor (GroupCell::GetEditable()) and draging
+   *   results in selecting text in EditorCell
+   * CLICK_TYPE_OUTPUT_SELECTION - we clicked in an output, we want selection to be confined to that
+   *   GroupCell's output. GC we first clicked in was stored in OnMouseMotion method
+   *   into m_clickInGC pointer.
+   */
   void ClickNDrag(wxPoint down, wxPoint up);
+  // Select all group cells inside the given rectangle;
+  void SelectGroupCells(wxPoint down, wxPoint up);
   void AdjustSize();
   void OnEraseBackground(wxEraseEvent& event) { }
   void CheckUnixCopy();
   void OnMouseMiddleUp(wxMouseEvent& event);
   void NumberSections();
   bool IsLesserGCType(int type, int comparedTo);
+  //! Is called if a action from the autocomplete menu is selected
   void OnComplete(wxCommandEvent &event);
   wxPoint m_down;
   wxPoint m_up;
@@ -305,9 +356,17 @@ class MathCtrl: public wxScrolledCanvas
     vertical line.
    */
   GroupCell *m_hCaretPosition;
-  //! The start for the selection when selecting group with the horizontally drawn cursor
+  /*! The start for the selection when selecting group with the horizontally drawn cursor
+
+    This cell does actually define weree the selection was started and therefore does not need 
+    to be above m_hCaretPositionEnd in the worksheet.
+   */
   GroupCell *m_hCaretPositionStart;
-  //! The start for the selection when selecting group with the horizontally drawn cursor
+  /*! The end of the selection when selecting group with the horizontally drawn cursor
+
+    This cell does actually define where the selection was ended and therefore does not need 
+    to be below m_hCaretPositionEnd in the worksheet.
+   */
   GroupCell *m_hCaretPositionEnd;
   bool m_leftDown;
   //! Do we want to automatically scroll to a cell as soon as it is being evaluated?
@@ -336,6 +395,8 @@ class MathCtrl: public wxScrolledCanvas
     cursor that is drawn as a horizontal line. 
    */
   EditorCell *m_activeCell;
+  EditorCell *m_cellMouseSelectionStartedIn;
+  EditorCell *m_cellKeyboardSelectionStartedIn;
   CellParser *m_selectionParser;
   bool m_switchDisplayCaret;
   /*! Is editing enabled?
@@ -356,6 +417,13 @@ class MathCtrl: public wxScrolledCanvas
 
 
 public:
+  /*! True = schedule an update of the table of contents
+    
+    used by UpdateTableOfContents() and the idle task.
+  */
+  bool m_scheduleUpdateToc;
+  
+  bool HCaretActive(){return m_hCaretActive;}
   /*! Can we merge the selected cells into one?
     
     \todo Does it make sense to make to allow the text of sections and image cells 
@@ -374,11 +442,6 @@ public:
   void TreeUndo_ClearBuffers();
 
   /*! The ids for all popup menu items.
-
-    \attention popid_complete_00 has to stay the last event in this enum
-    Since we want to be able to dynamically add events to popups
-    (for example for autocompletion) we need to make sure we know what 
-    the item with the lowest ID is we can assign dynamically.
   */
   enum PopIds{
     /*! The "copy" popup menu item was clicked
@@ -418,13 +481,9 @@ public:
     popid_insert_title,
     popid_insert_section,
     popid_insert_subsection,
-    /*! The first number that is open for dynamic ID assignment
-
-      If we want to add additional elements to a pop-up this is the 
-      lowest ID that is guaranteed to be free for this purpose.
-      \attention popid_must be the last id in this list
-     */
-    popid_complete_00
+    popid_insert_subsubsection,
+    menu_zoom_in,
+    menu_zoom_out
   };
   
   //! The constructor
@@ -469,8 +528,10 @@ public:
     the line is appended to m_last, instead.
   */
   void InsertLine(MathCell *newLine, bool forceNewLine = false);
-  void Recalculate(bool force = false);
-  void RecalculateForce();
+  void Recalculate(bool force = false);  
+  void RecalculateForce() {
+    Recalculate(true);
+  }
   /*! Empties the current document
 
     Used before opening a new file or when the "new" button is pressed.
@@ -487,7 +548,13 @@ public:
     return (m_activeCell != NULL && m_activeCell->CanCopy()) ||
            (m_selectionStart != NULL && m_selectionStart->GetType() == MC_TYPE_GROUP);
   }
+  //! Select the whole document
   void SelectAll();
+  //! Is at least one entire cell selected?
+  bool CellsSelected()
+    {
+      return((m_selectionStart != NULL) && (m_selectionEnd != NULL));
+    }
   /*! Delete a range of cells
   
     \param start The first cell to delete
@@ -528,6 +595,12 @@ public:
   //! Is it possible to delete the currently selected cells?
   bool CanDeleteSelection();
 
+  /*! Delete the currently active cell - or the cell above this one.
+
+    Used for the "delete current cell" shortcut.
+   */
+  void DeleteCurrentCell();
+
   //! Does it make sense to enable the "Play" button and the slider now? 
   bool CanAnimate() {
     return m_selectionStart != NULL && m_selectionStart == m_selectionEnd &&
@@ -552,8 +625,8 @@ public:
   bool CopyTeX();
   //! Copy a bitmap of the the current selection to the clipboard
   bool CopyBitmap();
-  bool CopyToFile(wxString file);
-  bool CopyToFile(wxString file, MathCell* start, MathCell* end, bool asData = false);
+  wxSize CopyToFile(wxString file);
+  wxSize CopyToFile(wxString file, MathCell* start, MathCell* end, bool asData = false,int scale=1);
   void CalculateReorderedCellIndices(MathCell *tree, int &cellIndex, std::vector<int>& cellMap);
   //! Export the file to an html document
   bool ExportToHTML(wxString file);
@@ -561,18 +634,21 @@ public:
   void ExportToMAC(wxTextFile& output, MathCell *tree, bool wxm, const std::vector<int>& cellMap, bool fixReorderedIndices);
   //! Export the file to a text file maxima's load command can read
   bool ExportToMAC(wxString file);
-  //! export to xml compatible file
-  bool ExportToWXMX(wxString file);	
+  /*! export to xml compatible file
+    \param file The file name
+    \param markAsSaved false means that this action doesn't clear the 
+                             worksheet's "modified" status.
+  */
+  bool ExportToWXMX(wxString file, bool markAsSaved = true);	
   //! export to a LaTeX file
   bool ExportToTeX(wxString file);
-  /*! Convert the current selection to a string
-
+  /*! Convert the current selection to a string 
     \param lb
      - true:  Include linebreaks
      - false: Remove linebreaks from the converted string
    */
   wxString GetString(bool lb = false);
-  MathCell* GetTree() { return m_tree; }
+  GroupCell* GetTree() { return m_tree; }
   /*! Return the first of the currently selected cells.
 
     NULL means: No cell is selected.
@@ -584,15 +660,25 @@ public:
   */
   MathCell* GetSelectionEnd() { return m_selectionEnd; }
   //! Select the cell sel
-  void SetSelection(MathCell* sel) { m_selectionStart = m_selectionEnd = sel; }
+  void SetSelection(MathCell* sel) { SetSelection(sel,sel); }
+  //! Select the cell range start-end
+  void SetSelection(MathCell* start,MathCell* end) { m_selectionStart = start;m_selectionEnd = end; }
   bool CanEdit();
   void EnableEdit(bool enable = true) { m_editingEnabled = enable; }
   bool ActivatePrevInput();
   bool ActivateNextInput(bool input = false);
+  //! Scrolls to the cursor
+  void ScrollToCaret();
   //! Scrolls to a given cell
   void ScrollToCell(MathCell *cell);
   //! Returns the cell the cursor that is drawn as a vertical line is in.
   EditorCell* GetActiveCell() { return m_activeCell; }
+  //! Is the point currently visible on the worksheet?
+  bool PointVisibleIs(wxPoint point);
+  //! Is the caret (hcaret or vcaret) currently visible on the worksheet?
+  bool CaretVisibleIs();
+
+  //! Scrolls to a point on the worksheet
   void ShowPoint(wxPoint point);
   void OnSetFocus(wxFocusEvent& event);
   void OnKillFocus(wxFocusEvent& event);
@@ -616,7 +702,7 @@ public:
   void SetHCaret(GroupCell *where, bool callRefresh = true); // call with false, when manually refreshing
   //! The cell the horizontal cursor is above. NULL means at the start of the document.
   GroupCell *GetHCaret();
-  //! Place the cursor into a new cell where the horizontal cursor is
+    //! Place the cursor into a new cell where the horizontal cursor is
   void OpenHCaret(wxString txt = wxEmptyString, int type = GC_TYPE_CODE);
   void ShowHCaret();
   /*! Is it possible to issue an undo in the currently selected cell? 
@@ -674,8 +760,6 @@ public:
   void AddSelectionToEvaluationQueue();
   //! Schedule this cell for evaluation
   void AddCellToEvaluationQueue(GroupCell* gc);
-  //! Unschedule all cells marked as to be evaluated
-  void ClearEvaluationQueue();
   //! The list of cells that have to be evaluated
   EvaluationQueue* m_evaluationQueue;
   // methods for folding
@@ -689,8 +773,7 @@ public:
   GroupCell *TearOutTree(GroupCell *start, GroupCell *end);
   // methods for zooming the document in and out
   double GetZoomFactor() { return m_zoomFactor; }
-  void SetZoomFactor(double newzoom, bool recalc = true) { m_zoomFactor = newzoom;
-    if (recalc) {RecalculateForce(); Refresh();} }
+  void SetZoomFactor(double newzoom, bool recalc = true);
   void CommentSelection();
   //! Called if the user is scrolling through the document.
   void OnScrollChanged(wxScrollEvent &ev);
@@ -703,12 +786,12 @@ public:
 
     Used by the find dialog.
    */
-  void Replace(wxString oldString, wxString newString);
+  void Replace(wxString oldString, wxString newString, bool ignoreCase);
   /*! Replace all ocourrences of a string
 
     Used by the find dialog.
    */
-  int ReplaceAll(wxString oldString, wxString newString);
+  int ReplaceAll(wxString oldString, wxString newString, bool ignoreCase);
   wxString GetInputAboveCaret();
   wxString GetOutputAboveCaret();
   bool LoadSymbols(wxString file) { return m_autocomplete.LoadSymbols(file); }
@@ -730,9 +813,21 @@ public:
   void QuestionAnswered();
   //! true = the last reply from maxima was a question
   bool m_questionPrompt;
-  //! Does the GroupCell cell points to contain the question currently asked by maxima?  
+  /*! Does maxima wait for the answer of a question?
+
+    \retval true = maxima waits for the answer of a question.
+  */
+  bool QuestionPending(){return m_questionPrompt;}
+
+    /*! Does maxima wait for the answer of a question?
+
+  */
+  void QuestionPending(bool pending){m_questionPrompt = pending;}
+//! Does the GroupCell cell points to contain the question currently asked by maxima?  
   bool GCContainsCurrentQuestion(GroupCell *cell);
-  /*! Move the cursor to the end of the question and if needed add a cell for user input
+  /*! Move the cursor to the question maxima currently asks and if needed add a cell for user input
+
+    \todo Currently scrolls to the GroupCell the question is in, not to the actual question.
    */
   void OpenQuestionCaret(wxString txt=wxT(""));
 
