@@ -48,8 +48,6 @@
 #include "SlideShowCell.h"
 #include "GroupCell.h"
 
-#define MAXLENGTH 50000
-
 MathParser::MathParser(wxString zipfile)
 {
   m_ParserStyle = MC_TYPE_DEFAULT;
@@ -83,6 +81,7 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
   bool hide = (node->GetAttribute(wxT("hide"), wxT("false")) == wxT("true")) ? true : false;
   // read (group)cell type
   wxString type = node->GetAttribute(wxT("type"), wxT("text"));
+  wxString sectioning_level = node->GetAttribute(wxT("sectioning_level"), wxT("0"));
 
   if (type == wxT("code")) {
     group = new GroupCell(GC_TYPE_CODE);
@@ -94,7 +93,11 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
         delete editor;
       }
       if (children->GetName() == wxT("output"))
-        group->AppendOutput(ParseTag(children->GetChildren()));
+      {
+        MathCell *tag = ParseTag(children->GetChildren());
+        if(tag != NULL)
+          group->AppendOutput(tag);
+      }
       children = children->GetNext();
     }
   }  else if (type == wxT("image")) {
@@ -127,7 +130,21 @@ MathCell* MathParser::ParseCellTag(wxXmlNode* node)
     else if (type == wxT("section"))
       group = new GroupCell(GC_TYPE_SECTION);
     else if (type == wxT("subsection"))
-      group = new GroupCell(GC_TYPE_SUBSECTION);
+    {
+      // We save subsubsections as subsections with a higher sectioning level:
+      // This makes them backwards-compatible in the way that they are displayed
+      // as subsections on old wxMaxima installations.
+      // A sectioning level of the value 0 means that the file is too old to
+      // provide a sectioning level.
+      if(sectioning_level != wxT("4"))
+        group = new GroupCell(GC_TYPE_SUBSECTION);
+      else
+        group = new GroupCell(GC_TYPE_SUBSUBSECTION);
+    }
+    else if (type == wxT("subsubsection"))
+    {
+      group = new GroupCell(GC_TYPE_SUBSUBSECTION);
+    }
     else
       return NULL;
 
@@ -180,6 +197,8 @@ MathCell* MathParser::ParseEditorTag(wxXmlNode* node)
     editor->SetType(MC_TYPE_SECTION);
   else if (type == wxT("subsection"))
     editor->SetType(MC_TYPE_SUBSECTION);
+  else if (type == wxT("subsubsection"))
+    editor->SetType(MC_TYPE_SUBSUBSECTION);
 
   wxString text = wxEmptyString;
   wxXmlNode *line = node->GetChildren();
@@ -392,11 +411,14 @@ MathCell* MathParser::ParseText(wxXmlNode* node, int style)
 	  int left= m_displayedDigits/3;
 	  if (left>30) left=30;
 	  
-	  str = str.Left(left) + wxString::Format(_(wxT("[%i digits]")), (int) str.Length() - 2 * left) + str.Right(left);
+	  str = str.Left(left) + wxString::Format(_("[%i digits]"), (int) str.Length() - 2 * left) + str.Right(left);
 	  //	  str = str.Left(left)+wxT("...");
 	}
     }
-    cell->SetType(m_ParserStyle);
+    if(style != TS_ERROR)
+      cell->SetType(m_ParserStyle);
+    else
+      cell->SetType(MC_TYPE_ERROR);
     cell->SetStyle(style);
     cell->SetHighlight(m_highlight);
     cell->SetValue(str);
@@ -636,10 +658,14 @@ MathCell* MathParser::ParseTag(wxXmlNode* node, bool all)
       }
       else if (tagName == wxT("t"))
       {          // Other text
+        TextStyle style = TS_DEFAULT;
+        if(node->GetAttribute(wxT("type")) == wxT("error"))
+          style = TS_ERROR;
+
         if (cell == NULL)
-          cell = ParseText(node->GetChildren(), TS_DEFAULT);
+          cell = ParseText(node->GetChildren(), style);
         else
-          cell->AppendCell(ParseText(node->GetChildren(), TS_DEFAULT));
+          cell->AppendCell(ParseText(node->GetChildren(), style));
       }
       else if (tagName == wxT("n"))
       {          // Numbers
@@ -972,9 +998,24 @@ MathCell* MathParser::ParseLine(wxString s, int style)
   MathCell* cell = NULL;
 
   wxConfigBase* config = wxConfig::Get();
-  bool showLong = false;
-  config->Read(wxT("showLong"), &showLong);
+  int showLength = 0;
+  config->Read(wxT("showLength"), &showLength);
 
+  switch(showLength)
+  {
+  case 0:
+    showLength = 50000;
+    break;
+  case 1:
+    showLength = 500000;
+    break;
+  case 2:
+    showLength = 5000000;
+    break;
+  case 3:
+    showLength = 0;
+    break;
+  }
   wxRegEx graph(wxT("[[:cntrl:]]"));
 
 #if wxUSE_UNICODE
@@ -983,7 +1024,7 @@ MathCell* MathParser::ParseLine(wxString s, int style)
   graph.Replace(&s, wxT("?"));
 #endif
 
-  if (s.Length() < MAXLENGTH || showLong)
+  if ((s.Length() < showLength) || (showLength=0))
   {
 
     wxXmlDocument xml;
