@@ -1,71 +1,60 @@
-///
-///  Copyright (C) 2004-2006 Andrej Vodopivec <andrejv@users.sourceforge.net>
-///
-///  This program is free software; you can redistribute it and/or modify
-///  it under the terms of the GNU General Public License as published by
-///  the Free Software Foundation; either version 2 of the License, or
-///  (at your option) any later version.
-///
-///  This program is distributed in the hope that it will be useful,
-///  but WITHOUT ANY WARRANTY; without even the implied warranty of
-///  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-///  GNU General Public License for more details.
-///
-///
-///  You should have received a copy of the GNU General Public License
-///  along with this program; if not, write to the Free Software
-///  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-///
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+//
+//  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
+//
+//  This program is free software; you can redistribute it and/or modify
+//  it under the terms of the GNU General Public License as published by
+//  the Free Software Foundation; either version 2 of the License, or
+//  (at your option) any later version.
+//
+//  This program is distributed in the hope that it will be useful,
+//  but WITHOUT ANY WARRANTY; without even the implied warranty of
+//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+//  GNU General Public License for more details.
+//
+//
+//  You should have received a copy of the GNU General Public License
+//  along with this program; if not, write to the Free Software
+//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//
 
 #include "Plot2dWiz.h"
 
 #include <wx/artprov.h>
 #include <wx/config.h>
 
-enum {
-  special,
-  combobox,
-  file_browse_2d,
-  parametric_plot,
-  discrete_plot
-};
-
-enum {
-  cartesian,
-  polar
-};
-
 Plot2DWiz::Plot2DWiz(wxWindow* parent, int id, const wxString& title,
                      const wxPoint& pos, const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  label_1 = new wxStaticText(this, -1, _("Plot 2D"));
   label_2 = new wxStaticText(this, -1, _("Expression(s):"));
   text_ctrl_1 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
-                              wxSize(250, -1));
+                              wxSize(300, -1));
   button_3 = new wxButton(this, special, _("&Special"));
   label_3 = new wxStaticText(this, -1, _("Variable:"));
   text_ctrl_2 = new BTextCtrl(this, -1, wxT("x"), wxDefaultPosition,
                               wxSize(40, -1));
-  label_4 = new wxStaticText(this, -1, _("from:"));
+  label_4 = new wxStaticText(this, -1, _("From:"));
   text_ctrl_3 = new BTextCtrl(this, -1, wxT("-5"), wxDefaultPosition,
                               wxSize(70, -1));
-  label_5 = new wxStaticText(this, -1, _("to:"));
+  label_5 = new wxStaticText(this, -1, _("To:"));
   text_ctrl_4 = new BTextCtrl(this, -1, wxT("5"), wxDefaultPosition,
                               wxSize(70, -1));
+  check_box_1 = new wxCheckBox(this, -1, _("logscale"));
   label_6 = new wxStaticText(this, -1, _("Variable:"));
   text_ctrl_5 = new BTextCtrl(this, -1, wxT("y"), wxDefaultPosition,
                               wxSize(40, -1), wxTE_READONLY);
-  label_7 = new wxStaticText(this, -1, _("from:"));
+  label_7 = new wxStaticText(this, -1, _("From:"));
   text_ctrl_6 = new BTextCtrl(this, -1, wxT("-5"), wxDefaultPosition,
                               wxSize(70, -1));
-  label_8 = new wxStaticText(this, -1, _("to:"));
+  label_8 = new wxStaticText(this, -1, _("To:"));
   text_ctrl_7 = new BTextCtrl(this, -1, wxT("5"), wxDefaultPosition,
                               wxSize(70, -1));
-  checkbox_1 = new wxCheckBox(this, -1, _("Send ranges to &gnuplot"));
+  check_box_2 = new wxCheckBox(this, -1, _("logscale"));
   label_9 = new wxStaticText(this, -1, _("Ticks:"));
   text_ctrl_8 = new wxSpinCtrl(this, -1, wxEmptyString, wxDefaultPosition,
-                               wxDefaultSize, wxSP_ARROW_KEYS, 0, 1000);
+                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 1000, 10);
+  text_ctrl_8->SetValue(10);
   label_10 = new wxStaticText(this, -1, _("Format:"));
   const wxString combo_box_1_choices[] =
     {
@@ -83,14 +72,12 @@ Plot2DWiz::Plot2DWiz(wxWindow* parent, int id, const wxString& title,
       wxT("set zeroaxis;"),
       wxT("set size ratio 1; set zeroaxis;"),
       wxT("set grid;"),
-      wxT("set polar; set zeroaxis;"),
-      wxT("set logscale y; set grid;"),
-      wxT("set logscale x; set grid;")
+      wxT("set polar; set zeroaxis;")
     };
   combo_box_2 = new wxComboBox(this, combobox, wxEmptyString, wxDefaultPosition,
-                               wxSize(280, -1), 6,
+                               wxSize(280, -1), 4,
                                combo_box_2_choices, wxCB_DROPDOWN);
-  label_12 = new wxStaticText(this, -1, _("Plot to file:"));
+  label_12 = new wxStaticText(this, -1, _("File:"));
   text_ctrl_9 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(280, -1));
   button_4 = new wxBitmapButton(this, file_browse_2d,
@@ -115,12 +102,10 @@ Plot2DWiz::Plot2DWiz(wxWindow* parent, int id, const wxString& title,
 void Plot2DWiz::set_properties()
 {
   SetTitle(_("Plot 2D"));
-  label_1->SetFont(wxFont(20, wxROMAN, wxITALIC, wxNORMAL, 0, wxEmptyString));
   text_ctrl_3->SetValue(wxT("-5"));
   text_ctrl_4->SetValue(wxT("5"));
   text_ctrl_6->SetValue(wxT("0"));
   text_ctrl_7->SetValue(wxT("0"));
-  text_ctrl_8->SetValue(wxT("10"));
 
   button_4->SetToolTip(_("Browse"));
 #if defined __WXMSW__
@@ -131,12 +116,17 @@ void Plot2DWiz::set_properties()
 
   int selection = 1;
   bool sendRanges = false;
+  bool logx = false, logy = false;
 
   wxConfig::Get()->Read(wxT("Wiz/Plot2D/format"), &selection);
   wxConfig::Get()->Read(wxT("Wiz/Plot2D/sendRanges"), &sendRanges);
+  wxConfig::Get()->Read(wxT("Wiz/Plot2D/logx"), &logx);
+  wxConfig::Get()->Read(wxT("Wiz/Plot2D/logy"), &logy);
+
+  check_box_1->SetValue(logx);
+  check_box_2->SetValue(logy);
 
   combo_box_1->SetSelection(selection);
-  checkbox_1->SetValue(sendRanges);
 
   text_ctrl_1->SetFocus();
 }
@@ -144,49 +134,48 @@ void Plot2DWiz::set_properties()
 
 void Plot2DWiz::do_layout()
 {
-  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 3, 3);
+  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 0, 0);
   wxBoxSizer* sizer_1 = new wxBoxSizer(wxHORIZONTAL);
-  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(7, 2, 3, 3);
+  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(7, 2, 0, 0);
   wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_3 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_4 = new wxBoxSizer(wxHORIZONTAL);
   wxBoxSizer* sizer_5 = new wxBoxSizer(wxHORIZONTAL);
-  grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_2->Add(text_ctrl_1, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_2->Add(button_3, 0, wxALL, 2);
+  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_2->Add(text_ctrl_1, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_2->Add(button_3, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_2, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_3->Add(text_ctrl_2, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(text_ctrl_3, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_3->Add(text_ctrl_4, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
+  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_3->Add(text_ctrl_2, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(text_ctrl_3, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(text_ctrl_4, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_3->Add(check_box_1, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer_2->Add(sizer_3, 1, 0, 0);
-  grid_sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_4->Add(text_ctrl_5, 0, wxALL, 2);
-  sizer_4->Add(label_7, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_4->Add(text_ctrl_6, 0, wxALL, 2);
-  sizer_4->Add(label_8, 0, wxALL | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  sizer_4->Add(text_ctrl_7, 0, wxALL, 2);
+  grid_sizer_2->Add(label_6, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_4->Add(text_ctrl_5, 0, wxALL, 5);
+  sizer_4->Add(label_7, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_4->Add(text_ctrl_6, 0, wxALL, 5);
+  sizer_4->Add(label_8, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  sizer_4->Add(text_ctrl_7, 0, wxALL, 5);
+  sizer_4->Add(check_box_2, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
   grid_sizer_2->Add(sizer_4, 1, wxEXPAND, 0);
-  grid_sizer_2->Add(10, 10);
-  grid_sizer_2->Add(checkbox_1, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(label_9, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_8, 0, wxALL, 2);
-  grid_sizer_2->Add(label_10, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(combo_box_1, 0, wxALL, 2);
-  grid_sizer_2->Add(label_11, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(combo_box_2, 0, wxALL, 2);
-  sizer_5->Add(text_ctrl_9, 0, wxALL | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_5->Add(button_4, 0, wxALL, 2);
-  grid_sizer_2->Add(label_12, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
+  grid_sizer_2->Add(label_9, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_8, 0, wxALL, 5);
+  grid_sizer_2->Add(label_10, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(combo_box_1, 0, wxALL, 5);
+  grid_sizer_2->Add(label_11, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(combo_box_2, 0, wxALL, 5);
+  sizer_5->Add(text_ctrl_9, 0, wxALL | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_5->Add(button_4, 0, wxALL, 5);
+  grid_sizer_2->Add(label_12, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
   grid_sizer_2->Add(sizer_5, 1, wxEXPAND, 0);
   grid_sizer_1->Add(grid_sizer_2, 1, wxEXPAND, 0);
   grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 2);
-  sizer_1->Add(button_1, 0, wxLEFT | wxRIGHT, 5);
-  sizer_1->Add(button_2, 0, wxLEFT | wxRIGHT, 5);
-  grid_sizer_1->Add(sizer_1, 1, wxALIGN_RIGHT | wxTOP | wxBOTTOM, 3);
+  sizer_1->Add(button_1, 0, wxALL, 5);
+  sizer_1->Add(button_2, 0, wxALL, 5);
+  grid_sizer_1->Add(sizer_1, 1, wxALIGN_RIGHT, 0);
   SetAutoLayout(true);
   SetSizer(grid_sizer_1);
   grid_sizer_1->Fit(this);
@@ -215,6 +204,9 @@ void Plot2DWiz::Parse(wxString s)
   unsigned int i = 0;
   wxString curr;
 
+  check_box_1->SetValue(false);
+  check_box_2->SetValue(false);
+
   s = s.SubString(7, s.Length());
   // Function to plot
   do
@@ -272,7 +264,7 @@ void Plot2DWiz::Parse(wxString s)
     {
       i++;
       curr = wxEmptyString;
-      while (i < s.Length() && s.GetChar(i) != ',')
+      while (i < s.Length() && s.GetChar(i) != ',' && s.GetChar(i) != ']')
       {
         curr += s.GetChar(i);
         i++;
@@ -299,7 +291,7 @@ void Plot2DWiz::Parse(wxString s)
         text_ctrl_7->SetValue(curr);
         i++;
       }
-      else if (curr == wxT("gnuplot_preamble"))
+      else if (curr == wxT("gnuplot_postamble"))
       {
         while (i < s.Length() && s.GetChar(i) != '"')
           i++;
@@ -338,6 +330,18 @@ void Plot2DWiz::Parse(wxString s)
         }
         text_ctrl_8->SetValue(curr);
       }
+      else if (curr == wxT("logx"))
+      {
+        check_box_1->SetValue(true);
+        while (i < s.Length() && s.GetChar(i) != ']')
+          i++;
+      }
+      else if (curr == wxT("logy"))
+      {
+        check_box_2->SetValue(true);
+        while (i < s.Length() && s.GetChar(i) != ']')
+          i++;
+      }
     }
     i++;
   }
@@ -347,27 +351,19 @@ wxString Plot2DWiz::GetValue()
 {
   wxString f = combo_box_1->GetValue();     // format
   wxString p = combo_box_2->GetValue();     // preamble
-  wxString s = wxT("plot2d([");             // result
+  wxString s;                               // result
   wxString x1 = text_ctrl_3->GetValue();
   wxString x2 = text_ctrl_4->GetValue();
   wxString y1 = text_ctrl_6->GetValue();
   wxString y2 = text_ctrl_7->GetValue();
   int t = text_ctrl_8->GetValue();          // Number of ticks
   wxString file = text_ctrl_9->GetValue();  // plot to file
-  wxString r;                               // range
-  if (checkbox_1->IsChecked())
-  {
-    r = wxT("set xrange [") + x1 + wxT(":") + x2 + wxT("];");
-    if (y1 != wxT("0") || y2 != wxT("0"))
-      r += wxT(" set yrange [") + y1 + wxT(":") + y2 + wxT("];");
-    if (p.Length())
-      p = r + wxT(" ") + p;
-    else
-      p = r;
-  }
 
-  s += text_ctrl_1->GetValue();
+  // Expression
+  s = wxT("plot2d([") + text_ctrl_1->GetValue();
   s += wxT("], [");
+
+  // x-range
   s += text_ctrl_2->GetValue();
   s += wxT(",");
   if (x1 != wxT("0") || x2 != wxT("0"))
@@ -377,22 +373,37 @@ wxString Plot2DWiz::GetValue()
   else
     s += wxT("-5,5");
   s += wxT("]");
+
+  // y-range
   if (y1 != wxT("0") || y2 != wxT("0"))
   {
     s += wxT(", [");
     s += text_ctrl_5->GetValue();
     s += wxT(",") + y1 + wxT(",") + y2 + wxT("]");
   }
+
+  // plot format
   if (f != _("default") && f != _("inline"))
     s += wxT(",\n [plot_format, ") + f + wxT("]");
+
+
+  // gnuplot_postamble
   if (p.Length() > 0)
-    s += wxT(",\n [gnuplot_preamble, \"") + p + wxT("\"]");
+    s += wxT(",\n [gnuplot_postamble, \"") + p + wxT("\"]");
   if (t != 10)
   {
     s += wxT(",\n [nticks,");
     s += wxString::Format(wxT("%d"), t);
     s += wxT("]");
   }
+
+  // check for logscales
+  if (check_box_1->IsChecked())
+    s += wxT(", [logx]");
+  if (check_box_2->IsChecked())
+    s += wxT(", [logy]");
+
+  // plot to file
   if (file.Length())
   {
     s += wxT(", [gnuplot_term, ps]");
@@ -404,16 +415,16 @@ wxString Plot2DWiz::GetValue()
       file = file + wxT(".eps");
     s += wxT(",\n [gnuplot_out_file, \"") + file + wxT("\"]");
   }
+  // inline?
   else if (f == _("inline"))
     s = wxT("wx") + s;
 
-  if (file.Length() == 0 && f == _("inline"))
-    s += wxT(");");
-  else
-    s += wxT(")$");
+  s += wxT(")$");
 
   wxConfig::Get()->Write(wxT("Wiz/Plot2D/format"), combo_box_1->GetSelection());
-  wxConfig::Get()->Write(wxT("Wiz/Plot2D/sendRanges"), checkbox_1->GetValue());
+  wxConfig::Get()->Write(wxT("Wiz/Plot2D/logx"), check_box_1->GetValue());
+  wxConfig::Get()->Write(wxT("Wiz/Plot2D/logy"), check_box_2->GetValue());
+
   return s;
 }
 
@@ -421,8 +432,8 @@ void Plot2DWiz::OnButton(wxCommandEvent& event)
 {
   wxMenu* popupMenu = new wxMenu();
 
-  popupMenu->Append(parametric_plot, wxT("Parametric plot"));
-  popupMenu->Append(discrete_plot, wxT("Discrete plot"));
+  popupMenu->Append(parametric_plot, _("Parametric plot"));
+  popupMenu->Append(discrete_plot, _("Discrete plot"));
 
   wxPoint pos = button_3->GetPosition();
   pos.y += button_3->GetRect().height;
@@ -490,7 +501,7 @@ void Plot2DWiz::OnFileBrowse(wxCommandEvent& event)
   wxString file = wxFileSelector(_("Save plot to file"), wxEmptyString,
                                  wxT("plot2d.eps"), wxT("eps"),
                                  _("Postscript file (*.eps)|*.eps|All|*"),
-                                 wxSAVE | wxOVERWRITE_PROMPT);
+                                 wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
   if (file.Length() > 0)
     text_ctrl_9->SetValue(file);
 }
@@ -513,7 +524,6 @@ Plot2DPar::Plot2DPar(wxWindow* parent, int id, const wxString& title,
                      const wxPoint& pos, const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  label_1 = new wxStaticText(this, -1, _("Parametric plot"));
   label_2 = new wxStaticText(this, -1, wxT("x = "));
   text_ctrl_1 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(230, -1));
@@ -523,15 +533,16 @@ Plot2DPar::Plot2DPar(wxWindow* parent, int id, const wxString& title,
   label_4 = new wxStaticText(this, -1, _("Variable:"));
   text_ctrl_3 = new BTextCtrl(this, -1, wxT("t"), wxDefaultPosition,
                               wxSize(40, -1));
-  label_5 = new wxStaticText(this, -1, _("from:"));
+  label_5 = new wxStaticText(this, -1, _("From:"));
   text_ctrl_4 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
-  label_6 = new wxStaticText(this, -1, _("to:"));
+  label_6 = new wxStaticText(this, -1, _("To:"));
   text_ctrl_5 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(70, -1));
   label_7 = new wxStaticText(this, -1, _("Ticks:"));
   spin_ctrl_1 = new wxSpinCtrl(this, -1, wxEmptyString, wxDefaultPosition,
-                               wxDefaultSize, wxSP_ARROW_KEYS, 0, 1000);
+                               wxSize(70, -1), wxSP_ARROW_KEYS, 0, 1000, 300);
+  spin_ctrl_1->SetValue(300);
   static_line_1 = new wxStaticLine(this, -1);
 #if defined __WXMSW__
   button_1 = new wxButton(this, wxID_OK, _("OK"));
@@ -548,10 +559,8 @@ Plot2DPar::Plot2DPar(wxWindow* parent, int id, const wxString& title,
 void Plot2DPar::set_properties()
 {
   SetTitle(_("Parametric plot"));
-  label_1->SetFont(wxFont(20, wxROMAN, wxITALIC, wxNORMAL, 0, wxEmptyString));
   text_ctrl_4->SetValue(wxT("-6"));
   text_ctrl_5->SetValue(wxT("6"));
-  spin_ctrl_1->SetValue(300);
 #if defined __WXMSW__
   button_1->SetDefault();
 #else
@@ -563,30 +572,29 @@ void Plot2DPar::set_properties()
 
 void Plot2DPar::do_layout()
 {
-  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 3, 3);
+  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 0, 0);
   wxBoxSizer* sizer_1 = new wxBoxSizer(wxHORIZONTAL);
-  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(4, 2, 3, 3);
+  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(4, 2, 0, 0);
   wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
-  grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_1, 0, wxALL | wxEXPAND, 2);
-  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_2, 0, wxALL | wxEXPAND, 2);
-  grid_sizer_2->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_1->Add(text_ctrl_3, 0, wxALL, 2);
-  sizer_1->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_1->Add(text_ctrl_4, 0, wxALL, 2);
-  sizer_1->Add(label_6, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 2);
-  sizer_1->Add(text_ctrl_5, 0, wxALL, 2);
+  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_1, 0, wxALL | wxEXPAND, 5);
+  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_2, 0, wxALL | wxEXPAND, 5);
+  grid_sizer_2->Add(label_4, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_1->Add(text_ctrl_3, 0, wxALL, 5);
+  sizer_1->Add(label_5, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_1->Add(text_ctrl_4, 0, wxALL, 5);
+  sizer_1->Add(label_6, 0, wxALL | wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL, 5);
+  sizer_1->Add(text_ctrl_5, 0, wxALL, 5);
   grid_sizer_2->Add(sizer_1, 1, 0, 0);
-  grid_sizer_2->Add(label_7, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(spin_ctrl_1, 0, wxALIGN_CENTER_VERTICAL | wxALL, 2);
+  grid_sizer_2->Add(label_7, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(spin_ctrl_1, 0, wxALIGN_CENTER_VERTICAL | wxALL, 5);
   grid_sizer_2->AddGrowableCol(1);
   grid_sizer_1->Add(grid_sizer_2, 1, wxEXPAND, 0);
-  grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 2);
-  sizer_2->Add(button_1, 0, wxLEFT | wxRIGHT, 5);
-  sizer_2->Add(button_2, 0, wxLEFT | wxRIGHT, 5);
-  grid_sizer_1->Add(sizer_2, 1, wxALIGN_RIGHT | wxTOP | wxBOTTOM, 3);
+  grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 5);
+  sizer_2->Add(button_1, 0, wxALL, 5);
+  sizer_2->Add(button_2, 0, wxALL, 5);
+  grid_sizer_1->Add(sizer_2, 1, wxALIGN_RIGHT, 0);
   SetAutoLayout(true);
   SetSizer(grid_sizer_1);
   grid_sizer_1->Fit(this);
@@ -623,7 +631,6 @@ Plot2DDiscrete::Plot2DDiscrete(wxWindow* parent, int id, const wxString& title,
                      const wxPoint& pos, const wxSize& size, long style):
     wxDialog(parent, id, title, pos, size, wxDEFAULT_DIALOG_STYLE)
 {
-  label_1 = new wxStaticText(this, -1, _("Discrete plot"));
   label_2 = new wxStaticText(this, -1, wxT("x = "));
   text_ctrl_1 = new BTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
                               wxSize(230, -1));
@@ -647,7 +654,6 @@ Plot2DDiscrete::Plot2DDiscrete(wxWindow* parent, int id, const wxString& title,
 void Plot2DDiscrete::set_properties()
 {
   SetTitle(_("Discrete plot"));
-  label_1->SetFont(wxFont(20, wxROMAN, wxITALIC, wxNORMAL, 0, wxEmptyString));
 #if defined __WXMSW__
   button_1->SetDefault();
 #else
@@ -662,20 +668,19 @@ void Plot2DDiscrete::set_properties()
 
 void Plot2DDiscrete::do_layout()
 {
-  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 3, 3);
-  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(2, 2, 3, 3);
+  wxFlexGridSizer* grid_sizer_1 = new wxFlexGridSizer(4, 1, 0, 0);
+  wxFlexGridSizer* grid_sizer_2 = new wxFlexGridSizer(2, 2, 0, 0);
   wxBoxSizer* sizer_2 = new wxBoxSizer(wxHORIZONTAL);
-  grid_sizer_1->Add(label_1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 2);
-  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_1, 0, wxALL | wxEXPAND, 2);
-  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 2);
-  grid_sizer_2->Add(text_ctrl_2, 0, wxALL | wxEXPAND, 2);
+  grid_sizer_2->Add(label_2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_1, 0, wxALL | wxEXPAND, 5);
+  grid_sizer_2->Add(label_3, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);
+  grid_sizer_2->Add(text_ctrl_2, 0, wxALL | wxEXPAND, 5);
   grid_sizer_2->AddGrowableCol(1);
   grid_sizer_1->Add(grid_sizer_2, 1, wxEXPAND, 0);
-  grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 2);
-  sizer_2->Add(button_1, 0, wxLEFT | wxRIGHT, 5);
-  sizer_2->Add(button_2, 0, wxLEFT | wxRIGHT, 5);
-  grid_sizer_1->Add(sizer_2, 1, wxALIGN_RIGHT | wxTOP | wxBOTTOM, 3);
+  grid_sizer_1->Add(static_line_1, 0, wxEXPAND | wxLEFT | wxRIGHT, 5);
+  sizer_2->Add(button_1, 0, wxALL, 5);
+  sizer_2->Add(button_2, 0, wxALL, 5);
+  grid_sizer_1->Add(sizer_2, 1, wxALIGN_RIGHT, 0);
   SetAutoLayout(true);
   SetSizer(grid_sizer_1);
   grid_sizer_1->Fit(this);
