@@ -188,9 +188,24 @@ void TextCell::Draw(CellParser& parser, wxPoint point, int fontsize)
 
     /// This is the default.
     else
-      dc.DrawText(m_text,
-                  point.x + SCALE_PX(MC_TEXT_PADDING, scale),
-                  point.y - m_realCenter + SCALE_PX(MC_TEXT_PADDING, scale));
+    {
+      switch(GetType())
+      {
+      case MC_TYPE_TEXT:
+        // TODO: Add markdown formatting for bold, italic and underlined here.
+        dc.DrawText(m_text,
+                    point.x + SCALE_PX(MC_TEXT_PADDING, scale),
+                    point.y - m_realCenter + SCALE_PX(MC_TEXT_PADDING, scale));
+        break;
+      case MC_TYPE_INPUT:
+        // This cell has already been drawn as an EditorCell => we don't repeat this action here.
+        break;
+      default:
+        dc.DrawText(m_text,
+                    point.x + SCALE_PX(MC_TEXT_PADDING, scale),
+                    point.y - m_realCenter + SCALE_PX(MC_TEXT_PADDING, scale));
+      }
+    }
   }
   MathCell::Draw(parser, point, fontsize);
 }
@@ -204,7 +219,10 @@ void TextCell::SetFont(CellParser& parser, int fontsize)
 
   if ((m_textStyle == TS_TITLE) ||
       (m_textStyle == TS_SECTION) ||
-      (m_textStyle == TS_SUBSECTION)) {
+      (m_textStyle == TS_SUBSECTION) ||
+      (m_textStyle == TS_SUBSUBSECTION)
+    )
+  {
     fontsize1 = parser.GetFontSize(m_textStyle);
     fontsize1 = (int) (((double)fontsize1) * scale + 0.5);
   }
@@ -231,10 +249,12 @@ void TextCell::SetFont(CellParser& parser, int fontsize)
                           m_fontname : parser.GetFontName(m_textStyle),
                       parser.GetFontEncoding()));
 
-  // Titles, sections, subsections - don't underline
+  // Titles, sections, subsections... - don't underline
   else if ((m_textStyle == TS_TITLE) ||
            (m_textStyle == TS_SECTION) ||
-           (m_textStyle == TS_SUBSECTION))
+           (m_textStyle == TS_SUBSECTION) ||
+           (m_textStyle == TS_SUBSUBSECTION)
+    )
     dc.SetFont(wxFont(fontsize1, wxFONTFAMILY_MODERN,
                       parser.IsItalic(m_textStyle),
                       parser.IsBold(m_textStyle),
@@ -274,8 +294,37 @@ wxString TextCell::ToString()
     text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
 #endif
   }
-  if (m_textStyle == TS_STRING)
+  switch(m_textStyle)
+  {
+  case TS_VARIABLE:
+  case TS_FUNCTION:
+    // The only way for variable or function names to contain quotes and
+    // characters that clearly represent operators is that these chars
+    // are quoted by a backslash: They cannot be quoted by quotation
+    // marks since maxima would'nt allow strings here.
+  {
+    // TODO: We could escape the - char. But we get false positives, then.
+    wxString charsNeedingQuotes("\\'\"()[]{}^+*/&§?:;=#<>$");
+    bool isOperator = true;
+    for(int i=0;i<m_text.Length();i++)
+    {
+      if((m_text[i]==wxT(' ')) || (charsNeedingQuotes.find(m_text[i])==wxNOT_FOUND))
+      {
+        isOperator = false;
+        break;
+      }
+    }
+
+    if(!isOperator)
+      for(int i=0;i<charsNeedingQuotes.Length();i++)
+        text.Replace(charsNeedingQuotes[i], wxT("\\") + wxString(charsNeedingQuotes[i]));
+  }
+    break;
+  case TS_STRING:
     text = wxT("\"") + text + wxT("\"");
+    break;
+  }
+
   return text;
 }
 
@@ -311,38 +360,9 @@ wxString TextCell::ToTeX()
   if (m_textStyle == TS_GREEK_CONSTANT)
   {
     if (m_text[0] != '%')
-      return wxT("%") + m_text;
+      return wxT("\\") + m_text;
     else
       return m_text;
-
-    if (m_text == wxT("%Alpha"))
-      return wxT("A");
-    else if (m_text == wxT("%Beta"))
-      return wxT("B");
-    else if (m_text == wxT("%Epsilon"))
-      return wxT("E");
-    else if (m_text == wxT("%Zeta"))
-      return wxT("Z");
-    else if (m_text == wxT("%Eta"))
-      return wxT("H");
-    else if (m_text == wxT("%Iota"))
-      return wxT("I");
-    else if (m_text == wxT("%Kappa"))
-      return wxT("K");
-    else if (m_text == wxT("%Mu"))
-      return wxT("M");
-    else if (m_text == wxT("%Nu"))
-      return wxT("N");
-    else if (m_text == wxT("%Omicron"))
-      return wxT("O");
-    else if (m_text == wxT("%Rho"))
-      return wxT("P");
-    else if (m_text == wxT("%Tau"))
-      return wxT("T");
-    else if (m_text == wxT("%Chi"))
-      return wxT("X");
-    else
-      return wxT("\\") + m_text.Mid(1);
   }
   
   if (m_textStyle == TS_SPECIAL_CONSTANT)
@@ -359,42 +379,61 @@ wxString TextCell::ToTeX()
       return m_text;
   }
 
-  
   wxString text = m_text;
   
-  if (m_type == MC_TYPE_LABEL)
-  {
-    
-    text = wxT("\\leqno{\\tt ") + m_text + wxT("}");
-    text.Replace(wxT("%"), wxT("\\%"));
+  if (m_textStyle == TS_LABEL)
+  {    
+    text = wxT("\\mathrm{\\tt ") + m_text + wxT("}\\quad ");
   }
   else
+  {
+    if (m_textStyle == TS_FUNCTION)
     {
-      if (m_textStyle == TS_FUNCTION)
-	if(m_text!=wxEmptyString)
-	  text = wxT("\\mathrm{") + m_text + wxT("}");
+      if(m_text!=wxEmptyString)
+        text = wxT("\\mathrm{") + m_text + wxT("}");
     }
-
+    else if (m_textStyle == TS_VARIABLE)
+    {
+      if(m_text.Length() > 1)
+        text = wxT("\\mathit{") + m_text + wxT("}");
+    }
+    else if (m_textStyle == TS_ERROR)
+    {
+      if(m_text.Length() > 1)
+        text = wxT("\\mbox{") + m_text + wxT("}");
+    }
+    else if (m_textStyle == TS_DEFAULT)
+    {
+      if(m_text.Length() > 2)
+        text = wxT("\\mbox{") + m_text + wxT("}");
+    }
+  }
+  
   if (
-      (m_textStyle != TS_FUNCTION) &&
-      (m_textStyle != TS_OUTDATED) &&
-      (m_textStyle != TS_VARIABLE) &&
-      (m_textStyle != TS_NUMBER)   &&
-      (m_textStyle != TS_GREEK_CONSTANT)   &&
-      (m_textStyle != TS_SPECIAL_CONSTANT)
-      )
+    (m_textStyle != TS_FUNCTION) &&
+    (m_textStyle != TS_OUTDATED) &&
+    (m_textStyle != TS_VARIABLE) &&
+    (m_textStyle != TS_NUMBER)   &&
+    (m_textStyle != TS_GREEK_CONSTANT)   &&
+    (m_textStyle != TS_SPECIAL_CONSTANT)
+    )
     text.Replace(wxT("^"), wxT("\\textasciicircum"));
-
+  
   if((m_textStyle == TS_DEFAULT)||(m_textStyle == TS_STRING))
+  {
+    if(m_text.Length()>1)
     {
-      if(m_text.Length()>1)
-	{
-	  if(((m_forceBreakLine)||(m_breakLine)))
-	    text=wxT("\\ifhmode\\\\\\fi\n")+text;
-	}
+      if(((m_forceBreakLine)||(m_breakLine)))
+        //text=wxT("\\ifhmode\\\\\\fi\n")+text;
+        text = wxT("\\mbox{}\\\\")+text;
+      if(m_textStyle != TS_DEFAULT)
+        text.Replace(wxT(" "), wxT("\\,"));
     }
+  }
   text.Replace(wxT("_"), wxT("\\_"));
+  text.Replace(wxT("$"), wxT("\\$"));
   text.Replace(wxT("%"), wxT("\\%"));
+  text.Replace(wxT("#"), wxT("\\#"));
 #if wxUSE_UNICODE
   text.Replace(wxT("\x2212"), wxT("-")); // unicode minus sign
 #endif
@@ -404,6 +443,7 @@ wxString TextCell::ToTeX()
 wxString TextCell::ToXML()
 {
   wxString tag;
+  wxString flags;
   if(m_isHidden)tag=_T("h");
     else
       switch(m_textStyle)
@@ -417,7 +457,10 @@ wxString TextCell::ToXML()
 	case TS_LABEL: tag=_T("lbl");break;
 	default: tag=_T("t");
 	}
-  
+
+  if(m_textStyle == TS_ERROR)
+    flags += wxT(" type=\"error\"");
+    
   wxString xmlstring = m_text;
   // convert it, so that the XML parser doesn't fail
   xmlstring.Replace(wxT("&"),  wxT("&amp;"));
@@ -426,7 +469,7 @@ wxString TextCell::ToXML()
   xmlstring.Replace(wxT("'"),  wxT("&apos;"));
   xmlstring.Replace(wxT("\""), wxT("&quot;"));
 
-  return _T("<") + tag + _T(">") + xmlstring + _T("</") + tag + _T(">");
+  return _T("<") + tag + flags +_T(">") + xmlstring + _T("</") + tag + _T(">");
 }
 
 wxString TextCell::GetDiffPart()
