@@ -41,23 +41,16 @@
 
 #define SOCKET_SIZE 1024
 #define DOCUMENT_VERSION_MAJOR 1
-#define DOCUMENT_VERSION_MINOR 2
+/*! The part of the .wxmx format version number that appears after the dot.
+  
+  - Updated to version 1.1 after user selectable animation-speeds were introduced:
+    Old wxMaxima versions play them back in the default speed instead but still
+    open the file.
+  - Bumped to version 1.2 after sub-subsections were introduced:
+    Old wxMaxima versions interpret them as subsections but still open the file.
+ */
+#define DOCUMENT_VERSION_MINOR 3
 
-class MyApp : public wxApp
-{
-public:
-  virtual bool OnInit();
-  wxLocale m_locale;
-  void NewWindow(wxString file = wxEmptyString);
-#if defined (__WXMAC__)
-  wxWindowList topLevelWindows;
-  void OnFileMenu(wxCommandEvent &ev);
-  virtual void MacNewFile();
-  virtual void MacOpenFile(const wxString& file);
-#endif
-};
-
-DECLARE_APP(MyApp)
 
 #ifndef __WXGTK__
 class MyAboutDialog : public wxDialog
@@ -76,12 +69,16 @@ public:
 class wxMaxima : public wxMaximaFrame
 {
 public:
-  
+  void CleanUp();                                  //!< shuts down server and client on exit
   //! An enum of individual IDs for all timers this class handles
   enum TimerIDs
   {
-    KEYBOARD_INACTIVITY_TIMER_ID,
-    AUTO_SAVE_TIMER_ID
+    //! The keyboard was inactive long enough that we can attempt an auto-save.
+    KEYBOARD_INACTIVITY_TIMER_ID, 
+    //! The time between two auto-saves has elapsed.
+    AUTO_SAVE_TIMER_ID,
+    //! We look if we got new data from maxima's stdout.
+    MAXIMA_STDOUT_POLL_ID
   };
 
   /*! A timer that determines when to do the next autosave;
@@ -99,6 +96,8 @@ public:
   bool m_autoSaveIntervalExpired;
   //! Is triggered when a timer this class is responsible for requires
   void OnTimerEvent(wxTimerEvent& event);
+  //! A timer that polls for output from the maxima process.
+  wxTimer m_maximaStdoutPollTimer;
 
   /*! The interval between auto-saves (in milliseconds). 
 
@@ -118,6 +117,10 @@ public:
   {
     m_openFile = file;
   }
+  void SetBatchMode(bool batch = false)
+  {
+    m_batchmode = batch;
+  }
   void StripComments(wxString& s);
   void SendMaxima(wxString s, bool history = false);
   void OpenFile(wxString file,
@@ -125,6 +128,16 @@ public:
   bool DocumentSaved() { return m_fileSaved; }
   void LoadImage(wxString file) { m_console->OpenHCaret(file, GC_TYPE_IMAGE); }
 private:
+  //! The number of output cells the current command has produced so far.
+  int m_outputCellsFromCurrentCommand;
+  //! The maximum number of lines per command we will display 
+  int m_maxOutputCellsPerCommand;
+  //! The number of consecutive unsucessfull attempts to connect to the maxima server
+  int m_unsuccessfullConnectionAttempts;
+  //! The current working directory maxima's file I/O is relative to.
+  wxString m_CWD;
+  //! Are we in batch mode?
+  bool m_batchmode;
   //! Can we display the "ready" prompt right now?
   bool m_ready;
   /*! A human-readable presentation of eventual unmatched-parenthesis type errors
@@ -133,10 +146,20 @@ private:
    */
   wxString GetUnmatchedParenthesisState(wxString text);
 protected:
+  //! Is called on start and whenever the configuration changes
+  void ConfigChanged();
   //! Called when the "Scroll to currently evaluated" button is pressed.
   void OnFollow(wxCommandEvent& event);
   void ShowCHMHelp(wxString helpfile,wxString keyword);
-  void ShowHTMLHelp(wxString helpfile,wxString keyword);
+  /*! Launches the HTML help browser
+
+    \param helpfile The name of the file the help browser has to be launched with
+    \param otherhelpfile We offer help for maxima and wxMaxima in separate manuals.
+                         This parameter contains the filename of the manual we aren't
+                         using currently so the help browser can open a tab containing
+                         this file.
+  */
+  void ShowHTMLHelp(wxString helpfile,wxString otherhelpfile,wxString keyword);
   void CheckForUpdates(bool reportUpToDate = false);
   void OnRecentDocument(wxCommandEvent& event);
   void OnIdle(wxIdleEvent& event);
@@ -180,9 +203,8 @@ protected:
   void ServerEvent(wxSocketEvent& event);          //!< server event: maxima connection
   /*! Is triggered on Input or disconnect from maxima
 
-    \todo An :lisp-quiet command causes the status line to be changed to "reading maxima
-    output". But since there will be no prompt afterwards the status line won't change 
-    back. This is for example triggered by opening a file. Is there a way to fix this?
+    The data we get from maxima is split into small packets we append to m_currentOutput 
+    until we got a full line we can display.
    */
   void ClientEvent(wxSocketEvent& event);
 
@@ -206,28 +228,51 @@ protected:
   void StructureDClick(wxCommandEvent& event);
   void OnInspectorEvent(wxCommandEvent& ev);
   void DumpProcessOutput();
+
+  //! Try to evaluate the next command for maxima that is in the evaluation queue
   void TryEvaluateNextInQueue();
+  //! Trigger execution of the evaluation queue
+  void TriggerEvaluation();
   void TryUpdateInspector();
 
   wxString ExtractFirstExpression(wxString entry);
   wxString GetDefaultEntry();
   bool StartServer();                              //!< starts the server
   bool StartMaxima();                              //!< starts maxima (uses getCommand)
-  void CleanUp();                                  //!< shuts down server and client on exit
   void OnClose(wxCloseEvent& event);               //!< close wxMaxima window
   wxString GetCommand(bool params = true);         //!< returns the command to start maxima
                                                    //    (uses guessConfiguration)
 
-  void ReadFirstPrompt();            //!< reads everything before first prompt
-  // setsup m_pid
-  void ReadPrompt();                 //!< reads prompts
-  void ReadMath();                   //!< reads output other than prompts
-  void ReadLispError();              //!< lisp errors (no prompt prefix/suffix)
-  void ReadLoadSymbols();            //!< functions after load command
+  //! Polls the stderr and stdout of maxima for input.
+  void ReadStdErr();
+  /*! Determines the process id of maxima from its initial output
+
+    This function does several things:
+     - it sets m_pid to the process id of maxima
+     - it discards all data until this point
+     - and it prepares the worksheet for editing.
+
+     \param data The string ReadFirstPrompt() does read its data from. 
+                  After leaving this function data is empty again.
+   */
+  void ReadFirstPrompt(wxString &data);
+  /* Reads the input and the output prompt from Maxima.
+   */
+  void ReadPrompt(wxString &data);
+  /* Reads the math cell's contents from Maxima.
+     
+     Math cells are enclosed between the tags \<mth\> and \</mth\>. If we 
+   */
+  void ReadMath(wxString &data);                   //!< reads the math that is contained in data
+  void ReadLispError(wxString &data);              //!< read lisp errors (no prompt prefix/suffix)
+  //! Reads autocompletion templates we get on load of a package
+  void ReadLoadSymbols(wxString &data);
 #ifndef __WXMSW__
-  void ReadProcessOutput();          //!< reads output of maxima command
+  //!< reads the output the maxima command sends to stdout
+  void ReadProcessOutput();                        
 #endif
-  bool SaveNecessary();              //!< Does this file contain anything worth saving?
+  //!< Does this file contain anything worth saving?
+  bool SaveNecessary();
 
   /*!
     This method is called once when maxima starts. It loads wxmathml.lisp
@@ -247,19 +292,34 @@ protected:
   bool OpenWXMXFile(wxString file, MathCtrl *document, bool clearDocument = true);
   GroupCell* CreateTreeFromXMLNode(wxXmlNode *xmlcells, wxString wxmxfilename = wxEmptyString);
   GroupCell* CreateTreeFromWXMCode(wxArrayString *wxmLines);
+  /*! Saves the current file
+
+    \param forceSave true means: Always ask for a file name before saving.
+   */
   bool SaveFile(bool forceSave = false);
   int SaveDocumentP();
+  //! Set the current working directory file I/O from maxima is relative to.
   void SetCWD(wxString file);
-
+  //! Get the current working directory file I/O from maxima is relative to.
+  wxString GetCWD()
+    {
+      return m_CWD;
+    }
   wxSocketBase *m_client;
   wxSocketServer *m_server;
   bool m_isConnected;
   bool m_isRunning;
   bool m_first;
+  //! The process id of maxima. Is determined by ReadFirstPrompt.
   long m_pid;
   wxProcess *m_process;
+  // The stdout of the maxima process
   wxInputStream *m_input;
+  // The stderr of the maxima process
+  wxInputStream *m_error;
   int m_port;
+  //! Are we currently saving the file?
+  bool m_saving;
   wxString m_currentOutput;
   wxString m_promptSuffix;
   wxString m_promptPrefix;
@@ -277,7 +337,7 @@ protected:
   bool m_fileSaved;
   bool m_variablesOK;
   wxString m_chmhelpFile;
-  wxString m_htmlhelpFile;
+  bool m_htmlHelpInitialized;
   wxString m_maximaVersion;
   wxString m_lispVersion;
 #if defined (__WXMSW__)
@@ -308,4 +368,30 @@ private:
 
 #endif
 
+class MyApp : public wxApp
+{
+public:
+  virtual bool OnInit();
+  wxLocale m_locale;
+  /*! Create a new window
+
+    \param file The file name
+    \param batchmode Do we want to execute the file and save it, but halt on error?
+   */
+  void NewWindow(wxString file = wxEmptyString,bool batchmode=false);
+  //! Is called by atExit and tries to close down the maxima process if wxMaxima has crashed.
+  static void Cleanup_Static();
+  //! A pointer to the currently running wxMaxima instance
+  static wxMaxima *m_frame;
+#if defined (__WXMAC__)
+  wxWindowList topLevelWindows;
+  void OnFileMenu(wxCommandEvent &ev);
+  virtual void MacNewFile();
+  virtual void MacOpenFile(const wxString& file);
+  
+#endif
+};
+
+DECLARE_APP(MyApp)
+
 #endif // WXMAXIMA_H
