@@ -35,6 +35,8 @@ wxMaximaFrame::wxMaximaFrame(wxWindow* parent, int id, const wxString& title,
   wxFrame(parent, id, title, pos, size, wxDEFAULT_FRAME_STYLE)
 
 {
+  m_EvaluationQueueLength = 0;
+  m_forceStatusbarUpdate = false;
   m_manager.SetManagedWindow(this);
   // console
   m_console = new MathCtrl(this, -1, wxDefaultPosition, wxDefaultSize);
@@ -46,22 +48,38 @@ wxMaximaFrame::wxMaximaFrame(wxWindow* parent, int id, const wxString& title,
   m_console->m_structure = new Structure(this, -1);
 
   SetupMenu();
-#if defined (__WXMSW__) || defined (__WXGTK20__) || defined (__WXMAC__)
-  m_console->m_mainToolBar = new ToolBar(CreateToolBar());
-  SetToolBar(m_console->m_mainToolBar->GetToolBar());
-#endif
 
   CreateStatusBar(2);
   int widths[] = { -1, 300 };
   SetStatusWidths(2, widths);
 
   m_StatusSaving = false;
+  // If we need to set the status manually for the first time using StatusMaximaBusy
+  // we first have to manually set the last state to something else.
+  m_StatusMaximaBusy = calculating;
   StatusMaximaBusy(waiting);
-  
+
+  // Add some shortcuts that aren't automatically set by menu entries.
 #if defined __WXMSW__
-  wxAcceleratorEntry entries[1];
-  entries[0].Set(wxACCEL_CTRL,  WXK_RETURN, menu_evaluate);
-  wxAcceleratorTable accel(1, entries);
+  wxAcceleratorEntry entries[7];
+  entries[0].Set(wxACCEL_CTRL,  WXK_TAB,    menu_autocomplete);
+  entries[1].Set(wxACCEL_CTRL,  WXK_SPACE,  menu_autocomplete);
+  entries[2].Set(wxACCEL_CTRL|wxACCEL_SHIFT,  WXK_TAB,    menu_autocomplete_templates);
+  entries[3].Set(wxACCEL_CTRL|wxACCEL_SHIFT,  WXK_SPACE,  menu_autocomplete_templates);
+  entries[4].Set(wxACCEL_CTRL,  wxT('+'),   MathCtrl::menu_zoom_in);
+  entries[5].Set(wxACCEL_CTRL,  wxT('-'),   MathCtrl::menu_zoom_out);
+  entries[6].Set(wxACCEL_CTRL,  WXK_RETURN, menu_evaluate);
+  wxAcceleratorTable accel(7, entries);
+  SetAcceleratorTable(accel);
+#else
+  wxAcceleratorEntry entries[6];
+  entries[0].Set(wxACCEL_CTRL,  WXK_TAB, menu_autocomplete);
+  entries[1].Set(wxACCEL_CTRL,  WXK_SPACE, menu_autocomplete);
+  entries[2].Set(wxACCEL_CTRL|wxACCEL_SHIFT,  WXK_TAB,    menu_autocomplete_templates);
+  entries[3].Set(wxACCEL_CTRL|wxACCEL_SHIFT,  WXK_SPACE,  menu_autocomplete_templates);
+  entries[4].Set(wxACCEL_CTRL,  wxT('+'), MathCtrl::menu_zoom_in);
+  entries[5].Set(wxACCEL_CTRL,  wxT('-'), MathCtrl::menu_zoom_out);
+  wxAcceleratorTable accel(6, entries);
   SetAcceleratorTable(accel);
 #endif
 
@@ -71,66 +89,115 @@ wxMaximaFrame::wxMaximaFrame(wxWindow* parent, int id, const wxString& title,
   m_console->SetFocus();
 }
 
-void wxMaximaFrame::StatusMaximaBusy(ToolbarStatus status)
-{  
-  if(!m_StatusSaving)
+void wxMaximaFrame::EvaluationQueueLength(int length)
+{
+  if(length != m_EvaluationQueueLength)
   {
-    switch(m_StatusMaximaBusy = status)
+    m_EvaluationQueueLength = length;
+    if(length>0)
+      SetStatusText(wxString::Format(_("%i cells in evaluation queue"),length),0);
+    else
+      SetStatusText(_("Welcome to wxMaxima"),0);
+  }
+}
+
+
+void wxMaximaFrame::StatusMaximaBusy(ToolbarStatus status)
+{
+  if((m_StatusMaximaBusy != status) || (m_forceStatusbarUpdate))
+  {
+    if(!m_StatusSaving)
     {
-    case userinput:	
-      m_MenuBar->Enable(menu_remove_output,false);
-      m_console->m_mainToolBar->ShowUserInputBitmap();
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    true);
-      SetStatusText(_("Maxima got a question"), 1);
-      break;
-    case waiting:
-      m_console->SetWorkingGroup(NULL);
-      m_console->QuestionAnswered();
-      m_MenuBar->Enable(menu_remove_output,true);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, false);
-      SetStatusText(_("Ready for user input"), 1);
-      // We don't evaluate any cell right now.
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,false);
-      m_console->m_mainToolBar->ShowFollowBitmap();
-      break;
-    case calculating:
-      m_MenuBar->Enable(menu_remove_output,false);
-      m_console->m_mainToolBar->ShowFollowBitmap();
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
-                                           m_console->ScrolledAwayFromEvaluation()
-        );
-      SetStatusText(_("Maxima is calculating"), 1);
-      break;
-    case transferring:
-      m_MenuBar->Enable(menu_remove_output,false);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
-                                           m_console->ScrolledAwayFromEvaluation()
-        );
-      SetStatusText(_("Reading Maxima output"), 1);
-      break;	
-    case parsing:
-      m_MenuBar->Enable(menu_remove_output,false);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
-      m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
-                                           m_console->ScrolledAwayFromEvaluation()
-        );
-      SetStatusText(_("Parsing output"), 1);
-      break;
+      switch(status)
+      {
+      case userinput:	
+        m_MenuBar->Enable(menu_remove_output,false);
+        if(m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    false);
+          m_console->m_mainToolBar->ShowUserInputBitmap();
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    true);
+        }
+        SetStatusText(_("Maxima got a question"), 1);
+        break;
+      case waiting:
+        m_console->SetWorkingGroup(NULL);
+        // If we evaluated a cell that produces no output we still want the
+        // cell to be unselected after evaluating it.
+        if(m_console->FollowEvaluation())
+          m_console->SetSelection(NULL);
+
+        m_MenuBar->Enable(menu_remove_output,true);
+        if (m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, false);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    false);
+          m_console->m_mainToolBar->ShowFollowBitmap();
+        }
+        SetStatusText(_("Ready for user input"), 1);
+        // We don't evaluate any cell right now.
+        break;
+      case calculating:
+        m_MenuBar->Enable(menu_remove_output,false);
+        if (m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    false);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
+                                               m_console->ScrolledAwayFromEvaluation()
+            );
+        }
+        SetStatusText(_("Maxima is calculating"), 1);
+        break;
+      case transferring:
+        m_MenuBar->Enable(menu_remove_output,false);
+        if (m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
+                                               m_console->ScrolledAwayFromEvaluation()
+            );
+        }
+        SetStatusText(_("Reading Maxima output"), 1);
+        break;	
+      case parsing:
+        m_MenuBar->Enable(menu_remove_output,false);
+        if (m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, true);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,
+                                               m_console->ScrolledAwayFromEvaluation()
+            );
+        }
+        SetStatusText(_("Parsing output"), 1);
+        break;
+      case disconnected:
+        m_MenuBar->Enable(menu_remove_output,false);
+        if (m_console->m_mainToolBar)
+        {
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_interrupt, false);
+          m_console->m_mainToolBar->EnableTool(ToolBar::tb_follow,    false);
+        }
+        SetStatusText(_("Not connected to maxima"), 1);
+        break;
+      }
     }
   }
+  m_StatusMaximaBusy = status;
+  m_forceStatusbarUpdate = false;
 }
 
 void wxMaximaFrame::StatusSaveStart()
 {
+  m_forceStatusbarUpdate = true;
   m_StatusSaving = true;
   SetStatusText(_("Saving..."), 1);
 }
 
 void wxMaximaFrame::StatusSaveFinished()
 {
+  m_forceStatusbarUpdate = true;
   m_StatusSaving = false;
   if(m_StatusMaximaBusy != waiting)
     StatusMaximaBusy(m_StatusMaximaBusy);
@@ -138,24 +205,48 @@ void wxMaximaFrame::StatusSaveFinished()
     SetStatusText(_("Saving successful."), 1);
 }
 
+void wxMaximaFrame::StatusExportStart()
+{
+  m_forceStatusbarUpdate = true;
+  m_StatusSaving = true;
+  SetStatusText(_("Exporting..."), 1);
+}
+
+void wxMaximaFrame::StatusExportFinished()
+{
+  m_forceStatusbarUpdate = true;
+  m_StatusSaving = false;
+  if(m_StatusMaximaBusy != waiting)
+    StatusMaximaBusy(m_StatusMaximaBusy);
+  else
+    SetStatusText(_("Export successful."), 1);
+}
+
 void wxMaximaFrame::StatusSaveFailed()
 {
+  m_forceStatusbarUpdate = true;
   m_StatusSaving = false;
   SetStatusText(_("Saving failed."), 1);
 }
 
+void wxMaximaFrame::StatusExportFailed()
+{
+  m_forceStatusbarUpdate = true;
+  m_StatusSaving = false;
+  SetStatusText(_("Export failed."), 1);
+}
+
 wxMaximaFrame::~wxMaximaFrame()
 {
   wxString perspective = m_manager.SavePerspective();
 
   wxConfig::Get()->Write(wxT("AUI/perspective"), perspective);
-  wxConfig::Get()->Write(wxT("AUI/toolbar"), (GetToolBar()->IsShown()));
-#if defined __WXMAC__
+#if defined __WXMAC__ || defined __WXMSW__
+  wxConfig::Get()->Write(wxT("AUI/toolbar"),
+                         GetToolBar() != NULL && GetToolBar()->IsShown());
 #else
+  wxConfig::Get()->Write(wxT("AUI/toolbar"), (m_console->m_mainToolBar!=NULL));
 #endif
-  /*
-    wxConfig::Get()->Write(wxT("AUI/toolbar"), (GetToolBar() != NULL));
-  */
   m_manager.UnInit();
   delete m_history;
   //  delete m_console->m_structure;
@@ -344,11 +435,14 @@ void wxMaximaFrame::SetupMenu()
   m_EditMenu->Append(menu_copy_to_file, _("Save Selection to Image..."),
                      _("Save selection from document to an image file"),
                      wxITEM_NORMAL);
-
   m_EditMenu->AppendSeparator();
-  APPEND_MENU_ITEM(m_EditMenu, menu_zoom_in, _("Zoom &In\tAlt-I"),
+  m_EditMenu->Append(MathCtrl::popid_comment_selection, _("Comment selection\tCtrl-/"),
+                     _("Comment out the currently selected text"),
+                     wxITEM_NORMAL);
+  m_EditMenu->AppendSeparator();
+  APPEND_MENU_ITEM(m_EditMenu, MathCtrl::menu_zoom_in, _("Zoom &In\tAlt-I"),
                    _("Zoom in 10%"), wxT("gtk-zoom-in"));
-  APPEND_MENU_ITEM(m_EditMenu, menu_zoom_out, _("Zoom Ou&t\tAlt-O"),
+  APPEND_MENU_ITEM(m_EditMenu, MathCtrl::menu_zoom_out, _("Zoom Ou&t\tAlt-O"),
                    _("Zoom out 10%"), wxT("gtk-zoom-out"));
   // zoom submenu
   m_Edit_Zoom_Sub = new wxMenu;
@@ -381,7 +475,7 @@ void wxMaximaFrame::SetupMenu()
                      _("Evaluate all visible cells in the document"), wxITEM_NORMAL);
   m_CellMenu->Append(menu_evaluate_all, _("Evaluate All Cells\tCtrl-Shift-R"),
                      _("Evaluate all cells in the document"), wxITEM_NORMAL);
-  m_CellMenu->Append(menu_evaluate_till_here, _("Evaluate Cells above this point\tCtrl-Shift-P"),
+  m_CellMenu->Append(ToolBar::tb_evaltillhere, _("Evaluate Cells above this point\tCtrl-Shift-P"),
                      _("Re-evaluate all cells above the one the cursor is in"), wxITEM_NORMAL);
 
   m_CellMenu->Append(menu_remove_output, _("Remove All Output"),
@@ -406,6 +500,8 @@ void wxMaximaFrame::SetupMenu()
                      _("Insert a new section cell"));
   m_CellMenu->Append(menu_add_subsection, _("Insert S&ubsection Cell\tCtrl-4"),
                      _("Insert a new subsection cell"));
+  m_CellMenu->Append(menu_add_subsubsection, _("Insert S&ubsubsection Cell\tCtrl-5"),
+                     _("Insert a new subsubsection cell"));
   m_CellMenu->Append(menu_add_pagebreak, _("Insert Page Break"),
                      _("Insert a page break"));
   m_CellMenu->Append(menu_insert_image, _("Insert Image..."),
@@ -437,11 +533,11 @@ void wxMaximaFrame::SetupMenu()
   m_Maxima_Panes_Sub->AppendSeparator();
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_math, _("General Math\tAlt-Shift-M"));
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_stats, _("Statistics\tAlt-Shift-S"));
-  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_history, _("Table of contents\tAlt-Shift-I"));
-  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_structure,  _("Structure\tAlt-Shift-T"));
+  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_history, _("History\tAlt-Shift-I"));
+  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_structure,  _("Table of contents\tAlt-Shift-T"));
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_format, _("Insert Cell\tAlt-Shift-C"));
   m_Maxima_Panes_Sub->AppendSeparator();
-  m_Maxima_Panes_Sub->AppendCheckItem(menu_show_toolbar, _("Toolbar\tAlt-Shift-T"));
+  m_Maxima_Panes_Sub->AppendCheckItem(menu_show_toolbar, _("Toolbar\tAlt-Shift-B"));
   m_MaximaMenu->Append(wxNewId(), _("Panes"), m_Maxima_Panes_Sub);
 
   m_MaximaMenu->AppendSeparator();
@@ -485,6 +581,9 @@ void wxMaximaFrame::SetupMenu()
                        wxITEM_NORMAL);
   m_MaximaMenu->Append(menu_texform, _("Display Te&X Form"),
                        _("Display last result in TeX form"), wxITEM_NORMAL);
+  m_MaximaMenu->AppendSeparator();
+  m_MaximaMenu->Append(menu_triggerEvaluation, _("Manually trigger evaluation"),
+                       _("If maxima ever finishes evaluating without wxMaxima realizing this this menu item can force wxMaxima to try to send commands to maxima again."), wxITEM_NORMAL);
   m_MenuBar->Append(m_MaximaMenu, _("&Maxima"));
 
   // Equations menu
@@ -745,8 +844,8 @@ void wxMaximaFrame::SetupMenu()
   m_NumericMenu->Append(menu_to_numer, _("To Numeri&c\tCtrl+Shift+N"),
                         _("Calculate numeric value of the last result"),
                         wxITEM_NORMAL);
-  m_NumericMenu->Append(menu_set_precision, _("Set &Precision..."),
-                        _("Set bigfloat precision"),
+  m_NumericMenu->Append(menu_set_precision, _("Set bigfloat &Precision..."),
+                        _("Set the precision for numbers that are defined as bigfloat. Such numbers can be generated by entering 1.5b12 or as bfloat(1.234)"),
                         wxITEM_NORMAL);
   m_MenuBar->Append(m_NumericMenu, _("&Numeric"));
 
@@ -916,9 +1015,11 @@ void wxMaximaFrame::ShowPane(Event id, bool show)
     m_manager.GetPane(wxT("history")).Show(show);
     break;
   case menu_pane_structure:
+  {
     m_manager.GetPane(wxT("structure")).Show(show);
-    m_console->m_structure->Update(m_console->GetTree());
-    break;
+    m_console->m_structure->Update(m_console->GetTree(),m_console->GetHCaret());
+  }
+  break;
   case menu_pane_stats:
     m_manager.GetPane(wxT("stats")).Show(show);
     break;
@@ -1050,6 +1151,7 @@ wxPanel *wxMaximaFrame::CreateFormatPane()
   grid->Add(new wxButton(panel, menu_format_text, _("Text")), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_title, _("Title")), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_subsection, _("Subsection")), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_subsubsection, _("Subsubsection")), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_section, _("Section")), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_image, _("Image")), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_pagebreak, _("Pagebreak")), 0, style, border);
@@ -1063,21 +1165,36 @@ wxPanel *wxMaximaFrame::CreateFormatPane()
 
 void wxMaximaFrame::ShowToolBar(bool show)
 {
+#if defined __WXMAC__ || defined __WXMSW__
   wxToolBar *tbar = GetToolBar();
+  if (tbar == NULL) {
+    tbar = CreateToolBar();
+    m_console->m_mainToolBar = new ToolBar(tbar);
+    SetToolBar(m_console->m_mainToolBar->GetToolBar());
+  }
   tbar->Show(show);
-#if defined __WXMAC__
+#if defined __WXMSW__
+  PostSizeEvent();
+#endif
 #else
-  /*
-    if (show) {
-    if (m_console->m_mainToolBar == NULL)
-    m_console->m_mainToolBar=new ToolBar(this,-1);
-    SetToolBar(m_console->m_mainToolBar);
+  if (show) {
+    if (m_console->m_mainToolBar == NULL) {
+      wxToolBar *tbar = CreateToolBar();
+      m_console->m_mainToolBar=new ToolBar(tbar);
     }
-    else
+    SetToolBar(m_console->m_mainToolBar->GetToolBar());
+  }
+  else
+  {
+    if(m_console->m_mainToolBar)
     {
-    m_console->m_mainToolBar->Destroy();
-    m_console->m_mainToolBar==NULL;
-    }*/
+      if(m_console->m_mainToolBar->GetToolBar())
+        delete m_console->m_mainToolBar->GetToolBar();
+      delete m_console->m_mainToolBar;
+      m_console->m_mainToolBar=NULL;
+      SetToolBar(NULL);
+    }
+  }
 #endif
 }
 
