@@ -87,6 +87,7 @@ public:
     socket_client_id,
     socket_server_id,
     input_line_id,
+    refresh_id,
     menu_new_id,
     menu_open_id,
     menu_batch_id,
@@ -175,6 +176,7 @@ public:
     menu_realpart,
     menu_imagpart,
     menu_subst,
+    menu_triggerEvaluation,
     button_factor_id,
     button_solve,
     button_solve_ode,
@@ -208,7 +210,6 @@ public:
     menu_add_path,
     menu_evaluate_all_visible,
     menu_evaluate_all,
-    menu_evaluate_till_here,
     menu_show_tip,
     menu_copy_from_console,
     menu_copy_tex_from_console,
@@ -222,8 +223,6 @@ public:
     menu_to_gamma,
     menu_texform,
     button_enter,
-    menu_zoom_in,
-    menu_zoom_out,
     menu_zoom_80,
     menu_zoom_100,
     menu_zoom_120,
@@ -237,6 +236,7 @@ public:
     menu_nouns,
     menu_evaluate,
     menu_add_comment,
+    menu_add_subsubsection,
     menu_add_subsection,
     menu_add_section,
     menu_add_title,
@@ -289,6 +289,7 @@ public:
     menu_stats_subsample,
     menu_format_code,
     menu_format_text,
+    menu_format_subsubsection,
     menu_format_subsection,
     menu_format_section,
     menu_format_title,
@@ -351,8 +352,15 @@ public:
     calculating,
     parsing,
     transferring,
-    userinput
+    userinput,
+    disconnected
   };
+
+  /*! Inform the user about the length of the evaluation queue.
+
+   */
+  void EvaluationQueueLength(int length);
+
   /*! Set the status according to if maxima is calculating 
 
     \param status
@@ -368,7 +376,15 @@ public:
   void StatusSaveFinished();
   //! Set the status to "Saving has failed"
   void StatusSaveFailed();
+  //! Set the status to "Maxima is exporting"
+  void StatusExportStart();
+  //! Set the status to "Maxima has finished exporting"
+  void StatusExportFinished();
+  //! Set the status to "Exporting has failed"
+  void StatusExportFailed();
 private:
+  //! The current length of the evaluation queue of commands we still need to send to maxima
+  int m_EvaluationQueueLength;
   //! True=We are currently saving.
   bool m_StatusSaving;
   //! The menu bar
@@ -420,6 +436,8 @@ protected:
   void SaveRecentDocuments();
   wxAuiManager m_manager;
 
+  //! true=force an update of the status bar at the next call of StatusMaximaBusy()
+  bool m_forceStatusbarUpdate;
   //! The worksheet itself
   MathCtrl*     m_console;
   //! The history pane
